package etc.ajax.charging;

import com.github.javaparser.utils.Pair;
import etc.ajax.AjaxClass;
import etc.ajax.AjaxLogic;
import etc.beans.user.SolutionSessionInfo;
import etc.caradmin.mvc.controller.charging.EnodeWebhookAPI;
import etc.caradmin.mvc.controller.charging.GetChargingDataEnode;
import etc.caradmin.mvc.util.WebUtils;
import okhttp3.MediaType;
import okhttp3.OkHttpClient;
import okhttp3.Request;
import okhttp3.RequestBody;
import okhttp3.Response;
import org.json.JSONObject;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.Objects;

/**
 * Klasse som henter link til  registrere en lader hos Enode.
 * @author Sindre Emil Vikre
 *
 */
@AjaxClass
public class ChargingEnodeLink extends AjaxLogic {

    //Object brukes til  hente string med enode milj som blir brukt.
    private final EnodeWebhookAPI enodeClass = new EnodeWebhookAPI();

    /**
     * doLogic kjrer frt nr Ajax klassen blir brukt. Logikken blir
     * styrt her.
     *
     * @param si
     * @param request
     * @param response
     * @throws IOException
     */
    @Override
    public void doLogic(
            final SolutionSessionInfo si, final HttpServletRequest request, final HttpServletResponse response
    ) throws IOException {


        //Fr enten en link eller en error tilbake fra getEnodeLink.
        Pair <String, String> enodePair = getEnodeLink(si);

        JSONObject obj = new JSONObject(); //Nytt json objekt som blir sendt tilbake til jsp filen.

        //Bestemmer nkkel og innhold basert p om man fikk en error eller en link fra funksjonen.
        obj = Objects.equals(enodePair.a, "success") ? obj.put("urlEnode", enodePair.b) : obj.put("message", enodePair.b);
        obj.put("status", enodePair.a);
        obj.write(response.getWriter()); //Sender objektet med info til jsp filen.
        response.getWriter().flush();


    }

    /**
     * getEnodeLink returnerer en url som brukes til  registrere en lader, eller en
     * melding hvis noe gikk galt.
     *
     * @param si
     * @return
     */
    public Pair<String, String> getEnodeLink(final SolutionSessionInfo si){

        //Fr access token til  gjre API kall hos Enode.
        GetChargingDataEnode enodeObj = new GetChargingDataEnode();
        String accessToken = enodeObj.getAccesToken();

        String customerSolution = si.getSolution(); //Henter navnet p kunden sin database.

        //Sjakker at token ikke er null.
        if(accessToken != null) {
            //Gjr et API kall for  f url fra Enode
            OkHttpClient client = new OkHttpClient().newBuilder()
                    .build();
            MediaType mediaType2 = MediaType.parse("application/x-www-form-urlencoded");
            RequestBody body2 = RequestBody.create(mediaType2, "vendorType=charger&redirectUri=https://localhost:8443/Autodb/x/SolutionSetup/i/224");
            Request request2 = new Request.Builder()
                    .url("https://enode-api." + enodeClass.getEnvironment() + ".enode.io/users/" + customerSolution + "/link")
                    .method("POST", body2)
                    .addHeader("Authorization", "Bearer " + accessToken)
                    .addHeader("Content-Type", "application/x-www-form-urlencoded")
                    .build();
            Response response2 = null;
            try {
                response2 = client.newCall(request2).execute();
            } catch(IOException e) {
                e.printStackTrace();
            }

            try {
                String jsonResLink = response2.body().string(); //Json string med body fra Enode API kallet.
                JSONObject jo2 = new JSONObject(jsonResLink);   //Gjr stringen om til et Json object.
                String linkUrl = jo2.getString("linkUrl"); //Henter url fra Json objectet

                return new Pair<String, String>("success", linkUrl); //Returnerer url og status
            } catch(Exception e) {
                String feedback = si.getMsg().t("Enode.CouldNotGetLink");
                return new Pair<String, String>("error", feedback); //Returnerer melding og status
            }

        }else{
            System.out.println("Failed to get Enode access token");
            String feedback = si.getMsg().t("Enode.CouldNotGetLink");
            return new Pair<String, String>("error", feedback);
        }
    }
}
