package etc.ajax.charging;

import etc.ajax.AjaxClass;
import etc.ajax.reports.DatahandlingContract;
import etc.ajax.reports.ModernJsonReportReader;
import etc.ajax.reports.PersonalInfoReportReader;
import etc.ajax.reports.ReportColumnHandler;
import etc.ajax.reports.ResultSetToJsonConsumer;
import etc.beans.roadmaintenance.RoadMaintenanceDrawShape;
import etc.beans.user.SolutionSessionInfo;
import etc.caradmin.beans.table.ModernReportFilter;
import etc.caradmin.beans.table.ReportParametersModern;
import etc.caradmin.io.CarAdminException;
import etc.caradmin.io.sql.SQLBuilder;
import etc.caradmin.mvc.controller.charging.ChargingHistory;
import etc.caradmin.mvc.controller.costunit.DimensionCostUnit;
import etc.caradmin.mvc.controller.fueling.FuelingHistory;
import etc.caradmin.mvc.util.WebUtils;
import etc.io.ClassFactory;
import etc.io.dimension.DimensionIO;
import etc.util.json.JSONArray;
import etc.util.json.JSONException;
import etc.util.json.JSONObject;
import etc.util.lang.Messages;
import org.apache.commons.lang3.StringUtils;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;

/**
 * Rapport for info om EL-billadinger.
 * @author Sindre Emil Vikre
 */
@AjaxClass
public class ChargingHistoryReport extends PersonalInfoReportReader {

    /**
     * doPersonalInfoLogic er startpunktet koden i denne klassen.
     *
     * @param si
     * @param request
     * @param response
     * @throws IOException
     */
    @Override
    protected void doPersonalInfoLogic(SolutionSessionInfo si, HttpServletRequest request, HttpServletResponse response) throws IOException {

        Messages m = si.getMsg();

        try {
            JSONObject jsonreport = createJsonReport(si, request, m); //Json object med rapportens innhold.
            if(jsonreport == null) {
                jsonreport = new JSONObject();
            }
            jsonreport.write(response.getWriter());
            response.getWriter().flush();

        } catch(JSONException | java.io.IOException e) {
            log.error("Feil ved henting av laderapport", e);
            response.setStatus(HttpServletResponse.SC_INTERNAL_SERVER_ERROR);
        }

    }

    /**
     * createJsonReport returnerer et json objekt med hva rapporten skal inneholde.
     *
     * @param si
     * @param request
     * @param m
     * @return
     */
    public JSONObject createJsonReport(SolutionSessionInfo si, HttpServletRequest request, Messages m) {
        initRequestInformation(request, si);

        ascDesc = request.getParameter("sortByAscending");
        sort = request.getParameter("sortByColumnName");

        String reportParameterName = "chargingParameters";

        ChargingHistory chargeCont = new ChargingHistory(si, reportParameterName);
        ReportParametersModern rp = chargeCont.getReportParameters(request);

        String start = rp.getPeriodStart().getFormatedDate(SQLBuilder.getDateFormat());
        String stopp = rp.getPeriodStop().getFormatedDate(SQLBuilder.getDateFormat());

        String Q = "WHERE ";

        //Queriet sorterer etter dato som er sendt til ajax filen
        if(start != null && stopp != null) {
            Q += " charging.date > '" + start + "' AND charging.date < '" + stopp + "'";
        }

        //Bestemmer hvilke kolonner i databasen som blir vist i rapporten.
        List<ReportColumnHandler> columnHandlers = new ArrayList<>();
        columnHandlers.add(ReportColumnHandler.of("id", ColumnDescriptor.Integer));
        columnHandlers.add(ReportColumnHandler.of("charging.chargerId", ColumnDescriptor.String));
        columnHandlers.add(ReportColumnHandler.of("charging.date", ColumnDescriptor.DateTime));
        columnHandlers.add(ReportColumnHandler.of("charging.brand", ColumnDescriptor.String));
        columnHandlers.add(ReportColumnHandler.of("charging.price", ColumnDescriptor.BigDecimal));
        columnHandlers.add(ReportColumnHandler.of("charging.kwh", ColumnDescriptor.BigDecimal));

        List<Object> parameterList = new ArrayList<>();

        return returnReportData(columnHandlers, "charging", Q, "charging.id", "id", "chargerId", si, true, parameterList.toArray());

    }
}