package etc.ajax.charging;

import etc.ajax.AjaxClass;
import etc.ajax.reports.PersonalInfoReportReader;
import etc.ajax.reports.ReportColumnHandler;
import etc.ajax.reports.ResultSetToJsonConsumer;
import etc.beans.user.SolutionSessionInfo;
import etc.caradmin.beans.table.ModernReportFilter;
import etc.caradmin.beans.table.ReportParametersModern;
import etc.caradmin.io.sql.SQLBuilder;
import etc.caradmin.mvc.controller.charging.ChargingTagOverview;
import etc.caradmin.mvc.util.WebUtils;
import etc.io.ClassFactory;
import etc.io.expence.ChargeIO;
import etc.util.json.JSONException;
import etc.util.json.JSONObject;
import org.apache.commons.lang3.StringUtils;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

/**
 * Rapportklasse for ladebrikke
 * @author Rihards D. U.
 */
@AjaxClass
public class ChargingTagReport extends PersonalInfoReportReader {
    @Override
    public void doPersonalInfoLogic(SolutionSessionInfo si, HttpServletRequest request, HttpServletResponse response) throws IOException {
        try {
            JSONObject jsonreport = createJsonReport(si, request);
            if (jsonreport == null) {
                jsonreport = new JSONObject();
            }
            jsonreport.write(response.getWriter());
            response.getWriter().flush();

        } catch (JSONException | java.io.IOException e) {
            log.error("Feil ved henting av ladebrikke-rapport", e);
            response.setStatus(HttpServletResponse.SC_INTERNAL_SERVER_ERROR);
        }
    }

    public JSONObject createJsonReport(SolutionSessionInfo si, HttpServletRequest request) {
        initRequestInformation(request, si);

        ascDesc = request.getParameter("sortByAscending");
        sort = request.getParameter("sortByColumnName");

        String reportParameterName = "chargeTagParameters";
        ChargingTagOverview chargingTagCont = new ChargingTagOverview(si, reportParameterName);
        ReportParametersModern rp = chargingTagCont.getReportParameters(request);

        String start = rp.getPeriodStart().getFormatedDate(SQLBuilder.getDateFormat());
        String stopp = rp.getPeriodStop().getFormatedDate(SQLBuilder.getDateFormat());

        String Q = "WHERE ";

        boolean whereSet = false;

        if(start != null && stopp != null) {
            Q += " chargingtag.created > '" + start + "' AND chargingtag.created < '" + stopp + "' ";
        }

        //liste over parametere til SQL
        List<Object> parameterList = new ArrayList<>();

        boolean isReload = WebUtils.hasParameter(request,"reload");

        if(isReload){
            if(WebUtils.hasParameter(request,"mainFilter")){
                String searchvalue = WebUtils.getStringOrDefault(request,null,"mainFilter");

                if(StringUtils.isBlank(searchvalue)) {
                    searchvalue = WebUtils.getStringOrDefault(request, null, "chargingTagSearch");
                }

                if(StringUtils.isNotBlank(searchvalue)) {
                    searchvalue = SQLBuilder.prepareGlobalSearch(searchvalue);

                    //Q = setWhere(Q, whereSet);
                    Q += "AND (general.carid LIKE ? ESCAPE '!' OR rfidNumber LIKE ? ESCAPE '!') ";
                    parameterList.add(searchvalue);
                    parameterList.add(searchvalue);
                }
            }
        }



        ModernReportFilter user = rp.getFilterByHtmlId("UserChoiseFilter");
        if(user != null && user.getSelectedKey() != null && !user.getSelectedKey().equalsIgnoreCase("all")){
            int userid = Integer.parseInt(user.getSelectedKey());
            Q = setWhere(Q, whereSet);
            Q += "chargingtagbinding.userid = ?";
            parameterList.add(userid);
        }

        joins.add("LEFT JOIN chargingtagbinding ON chargingtagbinding.tagid = chargingtag.id");
        joins.add("LEFT JOIN general ON general.carno = chargingtagbinding.assetno");
        //joins.add("LEFT JOIN customer ON customer.customerid = chargingtag.oilcompany");
        joins.add("LEFT JOIN user ON user.userid = chargingtag.createdby");

        List<ReportColumnHandler> columnHandlers = new ArrayList<>();
        columnHandlers.add(ReportColumnHandler.of("id", ColumnDescriptor.Integer));
        columnHandlers.add(ReportColumnHandler.of("rfidNumber", ColumnDescriptor.String));
        columnHandlers.add(ReportColumnHandler.of("created", ColumnDescriptor.Date));
        columnHandlers.add(ReportColumnHandler.of("createdby", ColumnDescriptor.Integer));
        columnHandlers.add(ReportColumnHandler.of("description", ColumnDescriptor.String));
        columnHandlers.add(ReportColumnHandler.of("user.firstname", ColumnDescriptor.String));
        columnHandlers.add(ReportColumnHandler.of("user.surname", ColumnDescriptor.String));


        String rowids = WebUtils.getStringOrDefault(request, null, "rowids");
        if(StringUtils.isNotBlank(rowids)){
            StringBuilder s = new StringBuilder("0");
            for (String r : rowids.split(",")) {
                if(StringUtils.isNotBlank(r)){
                    s.append(",").append(r);
                }
            }
            Q = setWhere(Q, whereSet);
            Q += " chargingtag.id IN("+s+")";
        }

        ResultSetToJsonConsumer createdByFormat = (rs, json) -> {
            if(json.has("user.firstname") && json.has("user.surname")){
                String f = json.getString("user.firstname");
                String s = json.getString("user.surname");
                if(StringUtils.isNotBlank(f) && StringUtils.isNotBlank(s)) {
                    json.put("createdby",f+" "+s);
                }
            }
        };
        columnHandlers.add(ReportColumnHandler.of("createdby", ColumnDescriptor.Composite, createdByFormat ) );

        // User and asset
        ChargeIO cio = ClassFactory.getChargeIO(si.getSqlFactory());
        ResultSetToJsonConsumer userAndAsset = (rs, json) -> {
            int tagId = json.getInt("id");
            List<JSONObject> bindings = cio.getBindingsForTag(tagId);
            if(bindings != null){
                StringBuilder cars = new StringBuilder();
                StringBuilder users = new StringBuilder();
                for(JSONObject j : bindings){
                    if(j.has("carid") && j.has("assetno")){
                        cars.append(j.getString("carid")).append(", ");
                    }
                    if(j.has("userid") && j.has("username")){
                        users.append(j.getString("username")).append(", ");
                    }
                }
                String car = cars.toString();
                if(StringUtils.isNotBlank(car)) {
                    car = car.substring(0, car.length() - 2);
                }
                json.put("assets",car);

                String usera = users.toString();
                if(StringUtils.isNotBlank(usera)) {
                    usera = usera.substring(0, usera.length() - 2);
                }
                json.put("users",usera);
            }

            // Charging Provider
            List<JSONObject> providerBindings = cio.getBindingsForProvider(tagId);
            if(providerBindings != null) {
                StringBuilder providers = new StringBuilder();
                for(JSONObject j : providerBindings){
                    if(j.has("name") && j.has("providerid")){
                        providers.append(j.getString("name")).append(", ");
                    }
                }
                String provider = providers.toString();
                if(StringUtils.isNotBlank(provider)) {
                    provider = provider.substring(0, provider.length() - 2);
                }
                json.put("providers",provider);
            }
        };
        columnHandlers.add(ReportColumnHandler.of("id", ColumnDescriptor.Composite, userAndAsset ) );

        return returnReportData(columnHandlers, "chargingtag", Q, "id", "id", "rfidNumber", si, true, parameterList.toArray());

    }

    private String setWhere(String q, boolean whereSet) {
        if(!whereSet){
            q += "WHERE ";
            whereSet = true;
        }else{
            q += " AND ";
        }
        return q;
    }
}
