package etc.ajax.charging;

import etc.ajax.AjaxClass;
import etc.ajax.AjaxLogic;
import etc.beans.user.SolutionSessionInfo;
import etc.caradmin.io.CarAdminException;
import etc.caradmin.mvc.controller.charging.ChargingTag;
import etc.caradmin.mvc.util.WebUtils;
import etc.io.ClassFactory;
import etc.io.expence.ChargeIO;
import etc.util.json.JSONException;
import etc.util.json.JSONObject;
import etc.util.schema.InputSchemaUtils;
import lombok.extern.log4j.Log4j2;
import org.apache.commons.lang3.StringUtils;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;

/**
 * Klasse for � lagre drivstoffkort
 *  @author Rihards D. U.
 */
@Log4j2
@AjaxClass
public class SaveChargingTag extends AjaxLogic {
    @Override
    public void doLogic(SolutionSessionInfo si, HttpServletRequest request, HttpServletResponse response) throws IOException {
        int id = WebUtils.getIntOrDefault(request, null, si, "chargingTagId");
        ChargeIO cio = ClassFactory.getChargeIO(si.getSqlFactory());
        try {
            ChargingTag ct = new ChargingTag();
            JSONObject ret = new JSONObject();
            if(id > 0){ //edit
                ct = cio.getChargingTagById(id);
                if(ct != null){
                    String singleField = WebUtils.getStringOrDefault(request, null, si, "singleField");
                    if(StringUtils.isNotBlank(singleField)){
                        ret = InputSchemaUtils.readObjectSingleFieldFromRequest(singleField, ct,request,null,si,null,null);
                    }
                    if(singleField.equalsIgnoreCase("rfidNumber")){ //hvis rfid er endret, sjekk om det er unikt
                        checkRfidNumber(si, cio, ret, ct);
                    }
                }

            }else{//Ny
                ret = InputSchemaUtils.readObjectFromRequest(ct, request, null, si, null);
                ct.setCreatedBy(si.getActingUser().getUserID());
                checkRfidNumber(si, cio, ret, ct);
            }

            //lagrer hvis det ikke er error
            if(!ret.has("error") && ct != null){
                cio.saveChargingTag(ct);
                if(ct.getId()>0){
                    ret.put("chargingTagId",ct.getId());
                }
            }

            ret.write(response.getWriter());
            response.getWriter().flush();
        } catch (JSONException | CarAdminException e) {
            log.error("Noe gikk feil ved lagring av ladebrikke",e);
        }


    }

    /**
     * Sjekker om kortnummeret er registrert fra f�r
     * @param si
     * @param cio
     * @param ret
     * @param ct
     * @throws CarAdminException
     */
    private void checkRfidNumber(SolutionSessionInfo si, ChargeIO cio, JSONObject ret, ChargingTag ct) throws CarAdminException {
        if(cio.getChargingTagByNumber(ct.getRfidNumber()) != null){
            ret.put("error",true);
            ret.put("customError","rfidNumber");
            ret.put("errorString",si.getMsg().t("ChargingTag.DuplicateCardError"));
        }
    }
}
