package etc.ajax.chargingTag;

import etc.ajax.chargingTagProviderParsing.ChargingTagProviderParser;
import etc.beans.user.SolutionSessionInfo;
import etc.caradmin.io.CarAdminException;
import etc.caradmin.mvc.controller.charging.ChargingProvider;
import etc.io.ClassFactory;
import etc.io.expence.ChargeIO;
import etc.util.json.JSONArray;
import etc.util.json.JSONObject;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Objects;

/**
 * Logger inn i ladeselskap ved hjelp av brukerinfo lagret i databasen, og
 * lagrer en validert token i databasen
 */
public class ChargingProviderLogin {

    /**
     * Sjekker om brukerinfo til ladeselskap lagret i databasen er riktig.
     * @param si
     * @return
     */
    public JSONArray getDbUserInfo(SolutionSessionInfo si) {
        JSONArray result = new JSONArray();
        ChargeIO cio = ClassFactory.getChargeIO(si.getSqlFactory());
        try {
            ArrayList<ChargingProvider> chargingProviders = cio.getChargingProviders();

            for(int i = 0; i < chargingProviders.size(); i++) {

                JSONObject a = new JSONObject();
                String provider = chargingProviders.get(i).getName();
                String username = chargingProviders.get(i).getUserName();
                String password = chargingProviders.get(i).getPassword();

                a.put("provider", provider);

                if(Objects.equals(username, "") || Objects.equals(password, "")) {
                    a.put("status", "empty");
                }else {
                    if(Objects.equals(dbTokenUpdate(cio, provider, username, password), "success")){
                        a.put("status", "registered");
                    }else {
                        a.put("status", "wrongPass");
                    }
                }
                result.put(a);
            }

        } catch(CarAdminException | IOException e) {
            e.printStackTrace();
        }
        return result;
    }

    /**
     * Oppdaterer validert token for forskjellige ladeselskap.
     * @param cio ChargeIO
     * @param ladeselskap ladeselskapet som får oppdatert token
     * @param username brukernavn brukt til å logge inn hos ladeselskap
     * @param password passord brukt til å logge inn hos ladeselskap
     * @return String "success" om brukeren fikk logget inn og fikk en ny validert token, "fail" om det failet.
     * @throws IOException
     */
    String dbTokenUpdate(ChargeIO cio, String ladeselskap, String username, String password) throws IOException {
        ChargingTagProviderParser loginParser = new ChargingTagProviderParser();

        String token = "";

        if(Objects.equals(ladeselskap, "Eviny")) {
            token = loginParser.bilkraftLogin(username, password);
        }

        if(Objects.equals(ladeselskap, "EON")) {
            token = loginParser.eonLogin(username, password);
        }

        if(token!=null && !token.isEmpty()){
            try {
                cio.updateChargingProviderToken(ladeselskap, token);
                return "success";
            } catch(CarAdminException e) {
                e.printStackTrace();
            }

        }

        return "fail";
    }
}
