package etc.ajax.chargingTagProviderParsing;

import com.fasterxml.jackson.databind.DeserializationFeature;
import com.fasterxml.jackson.databind.ObjectMapper;
import etc.caradmin.io.CarAdminException;
import etc.caradmin.mvc.controller.charging.ChargingTag;
import etc.io.expence.ChargeIO;
import etc.util.json.JSONArray;
import etc.util.json.JSONException;
import etc.util.json.JSONObject;
import okhttp3.MediaType;
import okhttp3.MultipartBody;
import okhttp3.OkHttpClient;
import okhttp3.Request;
import okhttp3.RequestBody;
import okhttp3.Response;
import org.apache.http.Header;
import org.apache.http.HttpResponse;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.ContentType;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.BasicResponseHandler;
import org.apache.http.impl.client.HttpClientBuilder;
import org.jsoup.Jsoup;
import org.jsoup.nodes.Document;
import org.jsoup.select.Elements;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Objects;

/**
 * Klasse som inneholder alle funksjoner brukt til � parse gjennom ladeselskapssider
 */
public class ChargingTagProviderParser {

    /**
     * Logger inn til bilkraft, og returnerer token for innsending av data til bilkraft(Eviny)
     *
     * @return validert token
     * @throws IOException
     */
    public String bilkraftLogin(String username, String password) throws IOException {

        String payload = "{\"userName\": \"" + username +"\", \"password\": \"" + password + "\"}";
        StringEntity entity = new StringEntity(payload,
                                               ContentType.APPLICATION_JSON);


        HttpClient httpClient = HttpClientBuilder.create().build();
        HttpPost httpRequest = new HttpPost("https://app.bilkraft.no/api/user/authenticate.json");
        httpRequest.setEntity(entity);


        HttpResponse httpResponse = httpClient.execute(httpRequest);
        String responseString = new BasicResponseHandler().handleResponse(httpResponse);
        String token = null;

        System.out.println("trying token");
        System.out.println("responseString: " + responseString);

        try {
            // Legger resultatet inn i et JSON Objekt
            JSONObject a = new JSONObject(responseString);

            if(a.has("token") && !a.isNull("token")) {
                token = a.getString("token");
            }

            System.out.println("token: " + token);
        } catch(JSONException e) {
            e.printStackTrace();
        }

        return token;
    }


    /**
     * Bruker token til å sende rfidnummer og bilnummer til bilkraft, for å registrere en ny ladebrikke.
     *
     * @param rfidnummer RFID nummeret som skal bli lagt inn i bilkraft
     * @param bilnummer Bilnummeret som skal bli lagt inn i bilkraft
     * @param token Token man får fra login funksjonen
     * @throws IOException
     * @return Resultatet fra API kallet
     */
    public String bilkraftPost(String rfidnummer, String bilnummer, String token) throws IOException {
        // Legger inn bilnummer og rfidnummer i payload
        String payloadJava = "{\"rfid\":{\"name\":\""+ bilnummer +"\",\"hexValue\": \"" + rfidnummer + "\"}}";
        StringEntity entityJava = new StringEntity(payloadJava,
                                                   ContentType.APPLICATION_JSON);

        // Lager en forespørsel til bilkraft
        HttpClient httpClientJava = HttpClientBuilder.create().build();
        HttpPost httpRequestJava = new HttpPost("https://app.bilkraft.no/api/user/rfid/add.json");
        httpRequestJava.setEntity(entityJava);

        // Legger inn validert token i header
        httpRequestJava.setHeader("x-token", token);

        // Kjører forespørselen og henter ut resultat
        HttpResponse responseJava = httpClientJava.execute(httpRequestJava);
        String responsestring = new BasicResponseHandler().handleResponse(responseJava);

        // Setter resultat lik 'FAIL'. Blir satt til 'SUCCESS' om vi fikk noe nyttig som resultat fra forespørselen over
        String result = "FAIL";

        try {
            // Legger resultatet inn i et JSON Objekt
            JSONObject a = new JSONObject(responsestring);

            // Leser resultatkode og returnerer resultat
            if(Objects.equals(a.getString("resultCode"), "SUCCESS")) {
                result = "SUCCESS";
                return result;
            }
            if(Objects.equals(a.getString("errorCode"), "RFID_IS_IN_USE")) {
                result = "RFID_IS_IN_USE";
                return result;
            }
        } catch(JSONException e) {
            e.printStackTrace();
        }
        return result;
    }

    /**
     * Logger inn til e.on, og returnerer token for innsending av data til e.on
     *
     * @return validert token
     * @throws IOException
     */
    public String eonLogin(String username, String password) throws IOException {


        //Henter SessionID f?r man logger inn
        HttpClient httpClient = HttpClientBuilder.create().build();

        HttpGet firstCookieRequest = new HttpGet("https://customer.eondrive.no/?login=true");

        org.apache.http.HttpResponse firstResponse = httpClient.execute(firstCookieRequest);

        Header[] firstCookies = firstResponse.getHeaders("Set-Cookie");
        String cookies = "";
        System.out.println("firstCookies: " + Arrays.toString(firstCookies));
        for(Header cookie : firstCookies) {
            cookies += cookie.getValue().split(";")[0];
            cookies += ";";
        }
        cookies = cookies.split(";")[2];

        //Logger inn og aktiverer dermed SessionID.
        OkHttpClient clientLogin = new OkHttpClient().newBuilder()
                .build();
        MediaType mediaType = MediaType.parse("text/plain");
        RequestBody bodylogin = new MultipartBody.Builder().setType(MultipartBody.FORM)
                .addFormDataPart("mobile",username)
                .addFormDataPart("password",password)
                .addFormDataPart("lang","nb")
                .build();
        Request request = new Request.Builder()
                .url("https://customer.eondrive.no/login.php")
                .method("POST", bodylogin)
                .addHeader("Cookie", cookies)
                .build();
        Response loginRes = clientLogin.newCall(request).execute();

        try {

            JSONObject resBody = new JSONObject(loginRes.body().string());

            if(resBody.has("result")){
                if(Objects.equals(resBody.getString("result"), "success")) {
                    return cookies;
                }
            }
        } catch(JSONException e) {
            return "";
        }
        return "";
    }

    /**
     * Bruker token til � sende rfidnummer til e.on, for � registrere en ny ladebrikke.
     *
     * @param rfidnummer
     * @param sessionID
     * @return JSONObject med rfidnummer og status om brikken ble lagret hos ladelevrandøren.
     * @throws IOException
     */
    public String eonPost(String rfidnummer, String sessionID) throws IOException {
        //Sender inn en ny ladebrikke
        OkHttpClient client = new OkHttpClient().newBuilder()
                .build();
        RequestBody body = new MultipartBody.Builder().setType(MultipartBody.FORM)
                .addFormDataPart("idtag",rfidnummer)
                .addFormDataPart("action","addIdtag")
                .build();
        Request request2 = new Request.Builder()
                .url("https://customer.eondrive.no/rfids.php")
                .method("POST", body)
                .addHeader("Cookie", sessionID)
                .build();
        Response response2 = client.newCall(request2).execute();
        String responsestring = response2.body().string();

        String result = "FAIL";
        try {

            JSONObject a = new JSONObject(responsestring);

            if(a.has("error")){
                result = "FAIL";
                return result;
            }

            if(a.has("message")) {
                if(Objects.equals(a.getString("message"), "Ny ladebrikke er lagret")) {
                    result="SUCCESS";
                    return result;
                }
            }
        } catch(JSONException e) {
            e.printStackTrace();
        }

        return result;
    }




    static public class ErrorCode{

    }
    public static class ErrorMessage{

    }
    public static class ExternalReference{

    }
    public static class Rfids{
        private String id;
        private String name;
        private String hexValue;
        private ExternalReference externalReference;
        private boolean isActive;

        public String getId(){
            return id;
        }
        public void setId(String input){
            this.id = input;
        }
        public String getName(){
            return name;
        }
        public void setName(String input){
            this.name = input;
        }
        public String getHexValue(){
            return hexValue;
        }
        public void setHexValue(String input){
            this.hexValue = input;
        }
        public ExternalReference getExternalReference(){
            return externalReference;
        }
        public void setExternalReference(ExternalReference input){
            this.externalReference = input;
        }
        public boolean getIsActive(){
            return isActive;
        }
        public void setIsActive(boolean input){
            this.isActive = input;
        }
    }
    public static class bilkraftTags{
        public String resultCode;
        public ErrorCode errorCode;
        public ErrorMessage errorMessage;
        public List<Rfids> rfids;

        public String getResultCode(){
            return resultCode;
        }
        public void setResultCode(String input){
            this.resultCode = input;
        }
        public ErrorCode getErrorCode(){
            return errorCode;
        }
        public void setErrorCode(ErrorCode input){
            this.errorCode = input;
        }
        public ErrorMessage getErrorMessage(){
            return errorMessage;
        }
        public void setErrorMessage(ErrorMessage input){
            this.errorMessage = input;
        }
        public List<Rfids> getRfids(){
            return rfids;
        }
        public void setRfids(List<Rfids> input){
            this.rfids = input;
        }
    }

    public static class bilkraftDeleteRes{
        private String resultCode;
        private String errorCode;
        private String errorMessage;

        public String getResultCode() {
            return resultCode;
        }

        public void setResultCode(final String resultCode) {
            this.resultCode = resultCode;
        }

        public String getErrorCode() {
            return errorCode;
        }

        public void setErrorCode(final String errorCode) {
            this.errorCode = errorCode;
        }

        public String getErrorMessage() {
            return errorMessage;
        }

        public void setErrorMessage(final String errorMessage) {
            this.errorMessage = errorMessage;
        }
    }

    /**
     * henter alle registrerte ladebrikker hos bilkraft.
     * @param token
     * @return bilkraftTags
     * @throws IOException
     */
    public bilkraftTags bilkraftGetLadebrikker(String token) throws IOException {

        OkHttpClient client = new OkHttpClient().newBuilder()
                .build();
        Request request = new Request.Builder()
                .url("https://app.bilkraft.no/api/user/rfid/list.json")
                .method("GET", null)
                .addHeader("x-token", token)
                .build();
        Response response = client.newCall(request).execute();

        ObjectMapper mapper = new ObjectMapper();
        mapper.configure(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES, false);
        bilkraftTags resObject = mapper.readValue(response.body().string(), bilkraftTags.class);
        if(resObject.resultCode.equals("SUCCESS")) {
            return resObject;
        }

        return null;
    }


    // Denne funksjonen er ikke brukt, men kan brukes for å parse Eon sin nettside, for å f.eks. hente ut alle ladebrikker registrert.
    // Kan være nyttig for å dobbeltsjekke at databaseinfo om registrerte rfidbrikker er riktig. Brukere kan legge inn rfid brikker hos ladeselskap
    // uten at CarAdmin får oppdatert sin database, derfor kan det være nyttig.
    public String eonGetLadebrikker(String token) throws IOException {

        OkHttpClient client = new OkHttpClient().newBuilder()
                .build();
        Request request = new Request.Builder()
                .url("https://customer.eondrive.no")
                .method("GET", null)
                .addHeader("Cookie", token)
                .build();
        Response response = client.newCall(request).execute();
        String responseString = response.body().string();
        Document doc = Jsoup.parse(responseString);
        Elements elements = doc.select("tr[id^=idtag-row-]");


        return elements.text();
    }

    /**
     * sletter en rfidbrikke fra bilkraft
     * @param rfidnummer
     * @param token
     * @return
     * @throws IOException
     */
    public String bilkraftDelete(String rfidnummer, String token) throws IOException {
        bilkraftTags ladebrikker = bilkraftGetLadebrikker(token);
        String id = "";
        String name = "";
        for(int i = 0; i < ladebrikker.rfids.size(); i++) {
            if(ladebrikker.rfids.get(i).hexValue.equals(rfidnummer)){
                id = ladebrikker.rfids.get(i).id;
                name = ladebrikker.rfids.get(i).name;
                break;
            }
        }

        if(id.equals("") || name.equals("")){
            return "fail";
        }

        String bodyString = "{\n" +
                "  \"rfid\": {\n" +
                "    \"id\": \""+ id +"\",\n" +
                "    \"name\": \""+name+"\",\n" +
                "    \"hexValue\": \""+rfidnummer+"\",\n" +
                "    \"externalReference\": null,\n" +
                "    \"isActive\": true\n" +
                "  }\n" +
                "}";

        OkHttpClient client = new OkHttpClient().newBuilder()
                .build();
        MediaType mediaType = MediaType.parse("application/json");
        RequestBody body = RequestBody.create(mediaType, bodyString);
        Request request = new Request.Builder()
                .url("https://app.bilkraft.no/api/user/rfid/remove.json")
                .method("DELETE", body)
                .addHeader("x-token", token)
                .addHeader("Content-Type", "application/json")
                .build();
        Response response = client.newCall(request).execute();

        ObjectMapper mapper = new ObjectMapper();
        mapper.configure(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES, false);
        String test = response.body().string();
        bilkraftDeleteRes resObject = mapper.readValue(test, bilkraftDeleteRes.class);

        if(resObject.resultCode.equals("SUCCESS")) {
            return "success";
        }

        return "fail";
    }


    /**
     * sletter en rfidbrikke hos eon
     * @param rfidnummer
     * @param token
     * @return
     * @throws IOException
     */
    public String eonDelete(String rfidnummer, String token) throws IOException {
        OkHttpClient client = new OkHttpClient().newBuilder()
                .build();
        MediaType mediaType = MediaType.parse("text/plain");
        RequestBody body = new MultipartBody.Builder().setType(MultipartBody.FORM)
                .addFormDataPart("idtag",rfidnummer)
                .addFormDataPart("action","deleteIdtag")
                .build();
        Request request = new Request.Builder()
                .url("https://customer.eondrive.no/rfids.php")
                .method("POST", body)
                .addHeader("Cookie", token)
                .build();
        Response response = client.newCall(request).execute();
        String responsestring = response.body().string();

        String result ="";
        try {

            JSONObject a = new JSONObject(responsestring);

            if(a.has("message")) {
                if(Objects.equals(a.getString("message"), "Ladebrikke er slettet")) {
                    result = "success";
                    return result;
                }
            }
        } catch(JSONException e) {
            e.printStackTrace();
        }

        return "fail";
    }
}
