package etc.api.restlet;

import etc.api.restlet.app.AppContactForm;
import etc.api.restlet.app.AppInitialize;
import etc.api.restlet.app.AppSessionFilter;
import etc.api.restlet.app.AppSolutionVerifier;
import etc.api.restlet.app.actions.RegisterActivity;
import etc.api.restlet.app.activeTrips.AppAdjustMileage;
import etc.api.restlet.app.activeTrips.AppChangeTracking;
import etc.api.restlet.app.activeTrips.AppExtendTrip;
import etc.api.restlet.app.activeTrips.AppGetFuelPin;
import etc.api.restlet.app.activeTrips.AppPreRegisterMileage;
import etc.api.restlet.app.activeTrips.AppRegisterFueling;
import etc.api.restlet.app.activeTrips.AppTripTypeToggle;
import etc.api.restlet.app.activeTrips.AppUpdateServiceDescription;
import etc.api.restlet.app.activeTrips.AppUpdateTripCategory;
import etc.api.restlet.app.activeTrips.GetActiveTrips;
import etc.api.restlet.app.auth.AppChangePassword;
import etc.api.restlet.app.auth.AppChangePasswordV2;
import etc.api.restlet.app.auth.AppLogin;
import etc.api.restlet.app.auth.AppLoginV2;
import etc.api.restlet.app.auth.AppLogout;
import etc.api.restlet.app.auth.AppRegisterMfa;
import etc.api.restlet.app.auth.AppTokenVerifier;
import etc.api.restlet.app.auth.AppVerifyMfa;
import etc.api.restlet.app.home.AppHomeButtons;
import etc.api.restlet.app.roadmaintenance.AppAcceptCallout;
import etc.api.restlet.app.roadmaintenance.AppChangeActivity;
import etc.api.restlet.app.roadmaintenance.AppInitRoadMaintenance;
import etc.api.restlet.app.roadmaintenance.AppPauseUnpause;
import etc.api.restlet.app.AppRecieveOfflineQueue;
import etc.api.restlet.app.roadmaintenance.AppRejectCallout;
import etc.api.restlet.app.roadmaintenance.AppSaveAlert;
import etc.api.restlet.app.roadmaintenance.AppStartStopRoute;
import etc.api.restlet.app.roadmaintenance.ConfirmCompletion;
import etc.api.restlet.app.roadmaintenance.GetMapAssets;
import etc.api.restlet.app.roadmaintenance.GetRMStatics;
import etc.api.restlet.app.roadmaintenance.GetTaskList;
import etc.api.restlet.app.roadmaintenance.LoadSingleRoute;
import etc.api.restlet.app.takeasset.AppSearchCallback;
import etc.api.restlet.app.takeasset.AppTakeAsset;
import etc.api.restlet.app.workshop.RespondToService;
import etc.api.restlet.asset.AssetMileage;
import etc.api.restlet.asset.CreateAsset;
import etc.api.restlet.asset.DeleteAsset;
import etc.api.restlet.asset.Service;
import etc.api.restlet.locker.ExternalLockerEvent;
import etc.api.restlet.outlook.ADSubscriptionHandler;
import etc.api.restlet.outlook.ContactSearch;
import etc.api.restlet.outlook.Contacts;
import etc.api.restlet.outlook.LeadSearch;
import etc.api.restlet.outlook.Leads;
import etc.api.restlet.outlook.OutlookAuthorize;
import etc.api.restlet.outlook.OutlookSubscriptionHandler;
import etc.api.restlet.outlook.UserData;
import etc.api.restlet.outlook.UserSync;
import etc.api.restlet.sync.ClientIpSyncFilter;
import etc.api.restlet.sync.SyncAssets;
import etc.api.restlet.sync.SyncUsers;
import etc.api.restlet.system.ressources.APILogin;
import etc.api.restlet.system.ressources.AppSolutionConnector;
import etc.api.restlet.system.ressources.InvoiceImport;
import etc.api.restlet.system.ressources.LastInvoiceImportDate;
import etc.api.restlet.system.ressources.ProdNoImport;
import etc.api.restlet.system.ressources.TripExport;
import etc.api.restlet.system.ressources.UserImport;
import etc.api.restlet.voucher.Voucher;
import etc.beans.api.ApiUser;
import etc.beans.api.ApiUserRole;
import etc.caradmin.io.CarAdminException;
import etc.caradmin.io.properties.SystemPreferences;
import etc.caradmin.mvc.controller.charging.EnodeWebhookAPI;
import etc.io.ClassFactory;
import org.apache.commons.collections.CollectionUtils;
import org.apache.logging.log4j.LogManager;
import org.jetbrains.annotations.NotNull;
import org.restlet.Application;
import org.restlet.Request;
import org.restlet.Response;
import org.restlet.Restlet;
import org.restlet.data.ChallengeScheme;
import org.restlet.engine.application.CorsFilter;
import org.restlet.routing.Router;
import org.restlet.routing.Template;
import org.restlet.security.ChallengeAuthenticator;
import org.restlet.security.MemoryRealm;
import org.restlet.security.Role;
import org.restlet.security.RoleAuthorizer;
import org.restlet.security.SecretVerifier;
import org.restlet.security.User;
import org.restlet.security.Verifier;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashSet;
import java.util.List;

/**
 * router klassen for API
 *
 * @author Sebastian
 */
public class RestletApplicationRouter extends Application {
    private ChallengeAuthenticator createAuthenticator() {
        ChallengeAuthenticator guard = new ChallengeAuthenticator(
                getContext(), ChallengeScheme.HTTP_BASIC, "realm");

        //Create in-memory users with roles
        MemoryRealm realm = new MemoryRealm();

        try {

            List<ApiUser> apiUsers = ClassFactory.getApiIO().getAllApiUsers(ClassFactory.getSharedSqlFactory());

            for(ApiUser apiUser : apiUsers) {
                String sol = apiUser.getSolution();
                String uname = apiUser.getUsername();
                String pw = apiUser.getSecret();

                User user = new User(uname, pw);
                user.setEmail(sol);
                realm.getUsers().add(user);

                if(!CollectionUtils.isEmpty(apiUser.getRoles())) {
                    for(ApiUserRole role : apiUser.getRoles()) {
                        realm.map(user, Role.get(this, role.name()));
                    }
                }
            }
        } catch(CarAdminException ignore) {
            //ignored
        }

        //Attach verifier to check authentication and enroler to determine roles
        guard.setVerifier(realm.getVerifier());
        guard.setEnroler(realm.getEnroler());

        return guard;
    }


    /**
     * {@inheritDoc}
     */
    @Override
    public Restlet createInboundRoot() {
        Router rootRouter = createRouter();

        LimiterService ls = new LimiterService(getContext());
        ls.setNext(rootRouter);

        return ls;
    }

    private Restlet createInternalSyncApiFilter(Router rootRouter) {
        return new ClientIpSyncFilter(getContext(), rootRouter);
    }

    private Restlet createOutlookOriginFilter(Router rootRouter) {
        try {
            SystemPreferences sPrefs = ClassFactory.getSystemPreferences(ClassFactory.getSharedSqlFactory());
            String originsAllow = sPrefs.read("api_origins_allow", "https://localhost");

            String[] originsArr = originsAllow.split(",");
            ArrayList<String> origins = new ArrayList<>(Arrays.asList(originsArr));

            CorsFilter corsFilter = new CorsFilter(getContext(), rootRouter);

            corsFilter.setAllowedOrigins(new HashSet<>(origins));
            corsFilter.setSkippingResourceForCorsOptions(true);

            return corsFilter;
        } catch(CarAdminException e) {
            LogManager.getLogger(this.getClass()).error("Klarte ikke finne ut om det er debug eller ei, " +
                                                                "API settes opp uten CORS");
            return null;
        }
    }

    @NotNull
    private Router createRouter() {
        Router publicRouter = new Router(getContext());
        publicRouter.setDefaultMatchingMode(Template.MODE_STARTS_WITH);
        publicRouter.setRoutingMode(Router.MODE_BEST_MATCH);

        /*
         * prov med curl:
         * curl -d "username=sebd&key=something" https://localhost/Api/Login -k			for loginpost med user form data
         * curl https://localhost/Api/ExportTrip/group/2016-05-23 -k					for getTip() med parameter
         */


        //App kobler seg mot serveren
        publicRouter.attach("/AppSolutionConnector", AppSolutionConnector.class);

        // Legger inn endepunkt for validering av Outlook-subscriptions
        publicRouter.attach("/OutlookSubscriptionHandler", OutlookSubscriptionHandler.class);

        // Legger inn endepunkt for validering av AD-subscriptions
        publicRouter.attach("/ADSubscriptionHandler", ADSubscriptionHandler.class);

        setupAppApi(publicRouter);

        setupInternalSyncApi(publicRouter);


        //Router for Enode api kall @author: Sindre Vikre
        setupEnodeRouter(publicRouter);

        //
        //Krev plogging for enkelte API-funksjoner
        Router privateRouter = setupPrivateApi(publicRouter);

        //sett opp Outlook-API
        setupOutlookApi(privateRouter);

        // Sett opp Sharebox-API
        setupShareboxApi(privateRouter);

        return publicRouter;

        /*   Eksempler fra Martin sin lsning
         router.attach("/login/{usertype}", LoginResource.class);
        router.attach("/users/{userId}/profilePicture", PublicProfilePictureResource.class);
        router.attach("/session/{key}-{id}/logout", LogoutResource.class);
        router.attach("/session/{key}-{id}/account", AccountResource.class);
        router.attach("/session/{key}-{id}/account/language", LanguageResource.class);
        router.attach("/session/{key}-{id}/account/email", EmailResource.class);
        router.attach("/session/{key}-{id}/account/password", PasswordResource.class);
        router.attach("/session/{key}-{id}/account/profilepicture", ProfilePictureResource.class);
        router.attach("/session/{key}-{id}/account/categories", CategoryOverviewResource.class);
        router.attach("/session/{key}-{id}/account/categories/{categoryId}", CategoryResource.class);
        router.attach("/session/{key}-{id}/account/activity/all/{count}", UserActivityResource.class);
        router.attach("/session/{key}-{id}/account/activity/notifications/{count}", UserNotificationResource.class);
        router.attach("/session/{key}-{id}/account/activity/{activityId}", SetEventResource.class);
        router.attach("/session/{key}-{id}/account/subscription", UserSubscriptionResource.class);
        router.attach("/session/{key}-{id}/account/subscription/setup", UserSubscriptionSetupResource.class);
        router.attach("/session/{key}-{id}/organization/members", MemberResource.class);
        router.attach("/session/{key}-{id}/organization/members/{userId}", MemberResource.class);
        router.attach("/session/{key}-{id}/organization/groups", UserGroupOverviewResource.class);
        router.attach("/session/{key}-{id}/organization/groups/{groupId}", UserGroupResource.class);
        router.attach("/session/{key}-{id}/organization/groups/{groupId}/members", UserGroupMembersResource.class);
        router.attach("/session/{key}-{id}/organization/groups/{groupId}/members/{userId}", UserGroupMembersResource.class);
        router.attach("/session/{key}-{id}/organization/groups/{groupId}/rights", UserGroupRightsManagementResource.class);
        router.attach("/session/{key}-{id}/organization/groups/{groupId}/rights/{categoryId}", UserGroupRightsManagementResource.class);
        router.attach("/session/{key}-{id}/contracts/canadd", AllowsMoreContractsResource.class);
        router.attach("/session/{key}-{id}/contracts", StoreContractResource.class);
        router.attach("/session/{key}-{id}/contracts/export/xlsx", RequestXLSXListResource.class);
        router.attach("/session/{key}-{id}/contracts/export/xls", RequestXLSListResource.class);
        router.attach("/session/{key}-{id}/contracts/export/csv", RequestCSVListResource.class);
        router.attach("/session/{key}-{id}/contracts/active", ActiveContractsResource.class);
        router.attach("/session/{key}-{id}/contracts/draft", DraftedContractsResource.class);
        router.attach("/session/{key}-{id}/contracts/expired", ExpiredContractsResource.class);
        router.attach("/session/{key}-{id}/contracts/calendar/{year}/{month}", ContractCalendarResource.class);
        router.attach("/session/{key}-{id}/contracts/expiring/{limit}", NextExpiringContractsResource.class);
        router.attach("/session/{key}-{id}/contracts/{contractId}/update", ContractResource.class);
        router.attach("/session/{key}-{id}/contracts/{contractId}", ContractResource.class);
        router.attach("/session/{key}-{id}/contracts/{contractId}/attachments", AttachmentOverviewResource.class);
        router.attach("/session/{key}-{id}/contracts/{contractId}/attachments/{attachmentId}", AttachmentResource.class);
        router.attach("/session/{key}-{id}/contracts/{contractId}/notifications", NotificationResource.class);
        router.attach("/session/{key}-{id}/contracts/{contractId}/notifications/{userId}", NotificationResource.class);
        router.attach("/session/{key}-{id}/contracts/{contractId}/notes", NoteResource.class);
        router.attach("/session/{key}-{id}/contracts/{contractId}/notes/{noteId}", NoteResource.class);
        router.attach("/session/{key}-{id}/contracts/{contractId}/export/pdf", RequestPDFResource.class);
        router.attach("/session/{key}-{id}/contracts/{contractId}/export/xlsx", RequestXLSXResource.class);
        router.attach("/session/{key}-{id}/contracts/{contractId}/export/xls", RequestXLSResource.class);
        router.attach("/session/{key}-{id}/contracts/{contractId}/export/csv", RequestCSVResource.class);
        router.attach("/session/{key}-{id}/contracts/{contractId}/rights", ContractRightsManagementResource.class);
        router.attach("/session/{key}-{id}/contracts/{contractId}/rights/current", CurrentContractRightsResource.class);
        router.attach("/session/{key}-{id}/contracts/{contractId}/rights/{userId}", ContractRightsManagementResource.class);
        router.attach("/session/{key}-{id}/devices/push/windows/{uri}", WindowsPushResource.class);
        router.attach("/session/{key}-{id}/devices/push/android/{uri}", AndroidPushResource.class);
        router.attach("/session/{key}-{id}/subscriptions", SubscriptionResource.class);
        router.attach("/session/{key}-{id}/system/settings", SystemSettingsResource.class);
        router.attach("/access/{key}-{id}", OneTimeAccessResource.class);
        router.attach("/activation/{key}-{id}", ActivationResource.class);
        router.attach("/languages", LocaleResource.class);
        router.attach("/languages/preferred", PreferredLocaleResource.class);
        router.attach("/batch/{key}", BatchResource.class);
        router.attach("/payment/activate/{token}", PaymentActivationResource.class);
         */
    }

    @NotNull
    private Router setupPrivateApi(final Router publicRouter) {
        ChallengeAuthenticator authenticator = createAuthenticator();

        publicRouter.attachDefault(authenticator);

        Router privateRouter = new Router(getContext());
        privateRouter.setDefaultMatchingMode(Template.MODE_STARTS_WITH);
        privateRouter.setRoutingMode(Router.MODE_BEST_MATCH);
        authenticator.setNext(privateRouter);



        //Eksport av turer
        privateRouter.attach("/{solutionUrl}/ExportTrip", TripExport.class);

        //Eksport av turer
        privateRouter.attach("/{solutionUrl}/AssetMileage", AssetMileage.class);

        //Import av bruker(e)
        privateRouter.attach("/{solutionUrl}/UserImport", UserImport.class);

        //Import av kostref/prodnummer
        privateRouter.attach("/{solutionUrl}/ProdNoImport", ProdNoImport.class);

        //lagring av kjrety med GPS
        privateRouter.attach("/{solutionUrl}/CreateAsset", CreateAsset.class);

        //fjerning av kjrety
        privateRouter.attach("/{solutionUrl}/DeleteAsset", DeleteAsset.class);

        // lagring av vedlikehold
        privateRouter.attach("/{solutionUrl}/Service", Service.class);

        // Innlogging
        privateRouter.attach("/Login", APILogin.class);

        //Import av faktura/bilag
        privateRouter.attach("/{solutionUrl}/InvoiceImport", InvoiceImport.class);
        //uthenting av siste fakturaimport
        privateRouter.attach("/{solutionUrl}/LastInvoiceImportDate", LastInvoiceImportDate.class);

        //eksport av bilag
        privateRouter.attach("/{solutionUrl}/Voucher", Voucher.class);

        //nullstill eksport-status p bilag
        privateRouter.attach("/${solutionUrl}/VoucherReset", Voucher.class);

        return privateRouter;
    }

    // @author: Sindre Vikre
    private void setupEnodeRouter(final Router publicRouter){

        publicRouter.attach("/EnodeWebhookAPI", EnodeWebhookAPI.class);

    }

    /**
     * Setter opp sharebox-API
     *
     * @param privateRouter router
     */
    private void setupShareboxApi(final Router privateRouter) {

        RoleAuthorizer authorizer = createShareboxAuthorizer();
        Router shareBoxRouter = new Router(getContext());
        privateRouter.attach("/{solutionUrl}/Sharebox", authorizer);
        authorizer.setNext(shareBoxRouter);

        shareBoxRouter.attach("/ShareboxLockerEvent", ExternalLockerEvent.class);
    }


    /**
     * Setter opp API-endepunkter for Outlook-API (som i praksis benyttes av CRM-tillegg i Outlook)
     *
     * @param privateRouter Router
     */
    private void setupOutlookApi(final Router privateRouter) {
        Router outlookRouter = new Router(getContext());
        var outFilter = createOutlookOriginFilter(outlookRouter);

        RoleAuthorizer authorizer = createOutlookAuthorizer();
        authorizer.setNext(outFilter);

        privateRouter.attach("/Outlook", authorizer);

        // Uthenting av brukerdata til Outlook med session
        outlookRouter.attach("/User", UserData.class);

        // Uthenting av leads til Outlook
        outlookRouter.attach("/Leads", Leads.class);

        // Uthenting av leads til Outlook
        outlookRouter.attach("/Leads/Search", LeadSearch.class);

        // Uthenting av kontakter til Outlook
        outlookRouter.attach("/Contacts", Contacts.class);

        // Uthenting av kontakter til Outlook
        outlookRouter.attach("/Contacts/Search", ContactSearch.class);

        // Verifiserer om bruker har synkronisert mot Outlook via CarAdmin
        outlookRouter.attach("/UserSync", UserSync.class);

        // Mottar autoriseringsdata etter at bruker har synkronisert mellom CarAdmin og Outlook
        outlookRouter.attach("/Authorize", OutlookAuthorize.class);
    }

    private void setupInternalSyncApi(final Router publicRouter) {
        Router syncRouter = new Router(getContext());
        var syncFilter = createInternalSyncApiFilter(syncRouter);
        publicRouter.attach("/SyncAPI", syncFilter);
        // Markering av Sync
        syncRouter.attach("/SyncUsers", SyncUsers.class);
        syncRouter.attach("/SyncAssets", SyncAssets.class);
    }

    private void setupAppApi(final Router publicRouter) {

        //prosess
        //-------- uten plogging:
        // ved gyldig app-secret:
        // verifiser kundenavn => car.caradmin.no/Lsning
        // logg p bruker -> fr tilbake en user-cookie

        // Verifiser lsningsnavn fra app
        publicRouter.attach("/AppAuth/VerifySolution", AppSolutionVerifier.class);

        // Innlogging fra app (gammel!!)
        publicRouter.attach("/AppAuth/Login", AppLogin.class);

        // Innlogging fra app
        publicRouter.attach("/AppAuth/Login/v2", AppLoginV2.class);

        // Passordbytte fra app (gammel!)
        publicRouter.attach("/AppAuth/ChangePassword", AppChangePassword.class);

        // Passordbytte fra app
        publicRouter.attach("/AppAuth/ChangePassword/v2", AppChangePasswordV2.class);

        // Utlogging fra app
        publicRouter.attach("/AppAuth/Logout", AppLogout.class);

        // Oppsett av MFA
        publicRouter.attach("/AppAuth/RegisterMfa", AppRegisterMfa.class);

        // Verifisering av MFA
        publicRouter.attach("/AppAuth/VerifyMfa", AppVerifyMfa.class);

        //innregistrering av utfylt kontaktskjema - krever systemtoken, ingen plogget bruker
        // Kontaktskjema (sender en mail)
        publicRouter.attach("/ContactForm", AppContactForm.class);

        // ---------- etter plogging:
        // Frst m app be om en session-key vha user-cookie
        // session-key har en utlpsdato og m sendes med i header p ALLE beskyttede kall

        Router protectedRouter = new Router(getContext());
        var appProtecter = new AppSessionFilter(getContext(), protectedRouter);
        publicRouter.attach("/App", appProtecter);

        // Oppstart av app (henter forsideknapper og aktive turer)
        protectedRouter.attach("/Initialize", AppInitialize.class);

        // Sjekk brukergyldighet fra app
        protectedRouter.attach("/VerifyToken/v2", AppTokenVerifier.class);

        // Henter menyoppsett (forside) til app
        protectedRouter.attach("/HomeButtons", AppHomeButtons.class);

        // Hent aktive turer for en bruker
        protectedRouter.attach("/Trips/GetActiveTrips/v2", GetActiveTrips.class);

        // Juster kilometerstand p en bil
        protectedRouter.attach("/Trips/AdjustMileage", AppAdjustMileage.class);

        // Endre sporingsmodus p en bil
        protectedRouter.attach("/Trips/ChangeTracking", AppChangeTracking.class);

        // Utvid en aktiv tur
        protectedRouter.attach("/Trips/ExtendTrip", AppExtendTrip.class);

        // Registrer en drivstoff-fylling
        protectedRouter.attach("/Trips/RegisterFueling", AppRegisterFueling.class);

        // Bytt mellom privat- og tjenestetur
        protectedRouter.attach("/Trips/ToggleTripType", AppTripTypeToggle.class);

        // Oppdater servicebeskrivelse p en bil
        protectedRouter.attach("/Trips/UpdateServiceDescription", AppUpdateServiceDescription.class);

        // Oppdater turtype p en bil
        protectedRouter.attach("/Trips/UpdateTripCategory", AppUpdateTripCategory.class);

        // Hent PIN-kode p drivstoffkort
        protectedRouter.attach("/Trips/GetFuelPin", AppGetFuelPin.class);

        // Start opp veivedlikehold i app
        protectedRouter.attach("/RoadMaintenance/Initialize", AppInitRoadMaintenance.class);

        // Hent arbeidsliste for en sjfr
        protectedRouter.attach("/RoadMaintenance/GetTaskList", GetTaskList.class);

        // Hent kjrety til rodekartet
        protectedRouter.attach("/RoadMaintenance/GetMapAssets", GetMapAssets.class);

        // Hent statiske data, som informasjon om hvilke rodesegmenter og snt som finnes
        protectedRouter.attach("/RoadMaintenance/GetStatics", GetRMStatics.class);

        // Forhndsregistrering av kilometerstand
        protectedRouter.attach("/Trips/PreRegMileage", AppPreRegisterMileage.class);

        // Legg til et veivarsel
        protectedRouter.attach("/RoadMaintenance/SaveAlert", AppSaveAlert.class);

        // Pause eller gjenoppta rodesporing
        protectedRouter.attach("/RoadMaintenance/PauseUnpauseTracking", AppPauseUnpause.class);

        // Start eller stopp en rode
        protectedRouter.attach("/RoadMaintenance/StartStopRoute", AppStartStopRoute.class);

        // Aksepter utkalling
        protectedRouter.attach("/RoadMaintenance/AcceptCallout", AppAcceptCallout.class);

        // Avvis utkalling
        protectedRouter.attach("/RoadMaintenance/RejectCallout", AppRejectCallout.class);

        // Bytt aktiviteter p et rodesegment
        protectedRouter.attach("/RoadMaintenance/ChangeActivity", AppChangeActivity.class);

        // G gjennom ting som har skjedd imens appen var offline
        protectedRouter.attach("/RoadMaintenance/RecieveOfflineQueue", AppRecieveOfflineQueue.class);

        // Hent informasjon om enkeltroder
        protectedRouter.attach("/RoadMaintenance/LoadSingleRoute", LoadSingleRoute.class);

        // Registrer et oppdrag som fullfrt
        protectedRouter.attach("/RoadMaintenance/ConfirmCompletion", ConfirmCompletion.class);

        // Registrer uttak via Bluetooth
        protectedRouter.attach("/TakeAsset", AppTakeAsset.class);

        // Send inn resultater av Bluetooth-sk
        protectedRouter.attach("/SearchCallback", AppSearchCallback.class);

        // Start eller stopp timefring
        protectedRouter.attach("/Activity", RegisterActivity.class);

        // G gjennom ting som har skjedd imens appen var offline
        protectedRouter.attach("/RecieveOfflineQueue", AppRecieveOfflineQueue.class);

        // Svar p innkalling til verksted
        protectedRouter.attach("/ServiceResponse", RespondToService.class);
    }

    /**
     * Lager authorizer for outlook som angir pkrevd rolle.
     *
     * @return Authorizer for outlook
     */
    private RoleAuthorizer createOutlookAuthorizer() {
        RoleAuthorizer roleAuth = new RoleAuthorizer();
        roleAuth.getAuthorizedRoles().add(Role.get(this, ApiUserRole.OUTLOOK.name()));
        return roleAuth;
    }

    /**
     * Lager authorizer for sharebox som angir pkrevd rolle.
     *
     * @return Authorizer for sharebox
     */
    private RoleAuthorizer createShareboxAuthorizer() {
        RoleAuthorizer roleAuth = new RoleAuthorizer();
        roleAuth.getAuthorizedRoles().add(Role.get(this, ApiUserRole.SHAREBOX.name()));
        return roleAuth;
    }
}

