package etc.caradmin.mvc.controller.admin;

import etc.ajax.chargingTag.ChargingProviderLogin;
import etc.beans.invoice.InvoiceDealBO;
import etc.beans.invoice.InvoiceReceiverBO;
import etc.beans.location.LocationBO;
import etc.beans.product.ProductBO;
import etc.beans.user.Rights;
import etc.beans.user.RoleAction;
import etc.beans.user.SolutionSessionInfo;
import etc.beans.user.UserGroup;
import etc.caradmin.asset.AssetSetting;
import etc.caradmin.asset.AssetType;
import etc.caradmin.beans.product.ProductFeatures;
import etc.caradmin.io.CarAdminException;
import etc.caradmin.io.properties.SystemPreferences;
import etc.caradmin.mvc.PageController;
import etc.caradmin.mvc.PageControllerAction;
import etc.caradmin.mvc.controller.Workshop.ExternalProductSupplier;
import etc.caradmin.mvc.controller.Workshop.ExternalProductWebService;
import etc.caradmin.mvc.controller.tolltag.TollTagSupplier;
import etc.caradmin.mvc.util.LabelValueFunctionList.LabelValueFunctionListUtil;
import etc.caradmin.mvc.util.valueprovider.StringValueProvider;
import etc.caradmin.mvc.util.valueprovider.TranslatedStringValueProvider;
import etc.caradmin.mvc.util.valueprovider.ValueProvider;
import etc.io.ClassFactory;
import etc.io.Workshop.WorkshopIO;
import etc.io.dimension.Dimension;
import etc.io.dimension.DimensionIO;
import etc.io.expence.ChargeIO;
import etc.io.expence.FuelIO;
import etc.io.location.LocationIO;
import etc.io.product.ProductIO;
import etc.io.product.ProductSettings;
import etc.io.tolltag.TollTagIO;
import etc.util.json.JSONArray;
import etc.util.json.JSONException;
import etc.util.json.JSONObject;
import etc.util.settings.SolutionPreferenceNames;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Objects;

/**
 * side som viser instillinger for API import
 *
 * @author Sebastian
 */
@PageControllerAction(rights = {RoleAction.ViewSolutionSetup})
public class SolutionSetup extends PageController {

    @Override
    protected void handleRequestImpl(HttpServletRequest request, HttpServletResponse response) {
        try {
            List<JSONObject> objectList = new ArrayList<>();

            final SystemPreferences solutionPreferences = ClassFactory.getSolutionPreferences(si.getSqlFactory());
            ProductFeatures activeFeatures = ProductSettings.getAllGlobal(si.getSqlFactory());

            LocationIO lio = ClassFactory.getLocationIO(si.getSqlFactory());

            addSettingsTag(request, objectList, solutionPreferences, lio, activeFeatures);

            if(Rights.hasRight(si, RoleAction.UpdateExternalAsset) && activeFeatures.isActivated(AssetSetting.EXTERNAL_ASSET)) {
                int extraCarLoc =
                        Integer.parseInt(
                                solutionPreferences.read(
                                        SolutionPreferenceNames.EXTRA_ASSET_STANDARD_LOCATION_ID,
                                        "0"));
                LocationBO extraLoc = lio.getLocation(extraCarLoc);

                String exLocLab = m.t("Settings.extraAssetStandardLocationId");
                String exLocVal = (extraLoc == null) ? "" : extraLoc.getName();
                objectList.add(LabelValueFunctionListUtil
                                       .getItem(exLocLab, exLocVal, "/images/newdesign/edit_104_darkgray.png", "getDialog('ExtraAssetStdLoc')"));
            }

            request.setAttribute("apisettingslist", objectList);


            if(Rights.hasRight(si, RoleAction.EditFuelCards)
                    && activeFeatures.isActivated(AssetSetting.FUEL_CARD)) {
                addFuelingTag(request, si);
            }

            //@author: Elliot Sveum Torp
            if(Rights.hasRight(si, RoleAction.EditFuelCards)
                    && activeFeatures.isActivated(AssetSetting.FUEL_CARD)) {
                addChargingProvider(request, si);
            }

            if(Rights.hasRight(si, RoleAction.EditRoadMaintenance)
                    && activeFeatures.isActivated(AssetSetting.ROADMAINTENANCE)) {
                addRoadMaintenance(request);
            }

            if(Rights.hasRight(si, RoleAction.EditTollTags) && activeFeatures.isActivated(AssetSetting.TOLL_TAGS)) {
                addTollTag(request, si);
            }

            if(Rights.hasRight(si, RoleAction.EditEconomyDimension) && activeFeatures.isActivated(AssetSetting.EXTENDED_DIMENSIONS)){
                addDimensionTripOrderTag(request);
            }
            if(Rights.hasRight(si, RoleAction.EditWorkshopOrders) && activeFeatures.isActivated(AssetSetting.WORKSHOPBOOK) && activeFeatures.isActivated(AssetSetting.EXTERNAL_PRODUCT_CATALOG)) {
                    addWorkShopExternalProductCatalog(request);
            }
        } catch(CarAdminException e) {
            getLogger().error("Error ved lasting Api settings", e);
            response.setStatus(HttpServletResponse.SC_INTERNAL_SERVER_ERROR);
        }
    }

    private void addWorkShopExternalProductCatalog(final HttpServletRequest request) throws CarAdminException {
        List<JSONObject> espList = new ArrayList<>();
        WorkshopIO wsio = ClassFactory.getWorkshopIO(si.getSqlFactory());
        List<ExternalProductSupplier> esp = wsio.getExternalProductSuppliers();
        List<ExternalProductWebService> vals = new ArrayList<>();
        for(ExternalProductWebService epc : ExternalProductWebService.values()){
            vals.add(epc);
        }

        if(esp != null && !esp.isEmpty()) {
            for(ExternalProductSupplier f : esp) {
                vals.remove(f.getEpc());
                espList.add(
                    LabelValueFunctionListUtil.getItem(
                        f.getEpc().name(), "",
                        "/images/newdesign/delete.png",
                        "removeExternalProductSupplierOnAPI('"
                                + f.getId()
                                + "')"));
            }
        }

        JSONObject head;
        if(vals.isEmpty()){
            head = LabelValueFunctionListUtil.getHeaderItem(
                    si.getMsg().t("Api.ExternalProductCatalogWorkshop"));
        }else {
            head = LabelValueFunctionListUtil.getHeaderItem(
                    si.getMsg().t("Api.ExternalProductCatalogWorkshop"),
                    "",
                    "/images/newdesign/plus.png",
                    "getDialog('ApiSettingsExternalProductCatalog')");
        }
        request.setAttribute("ExternalProductCatalogHeading",head);
        request.setAttribute("espList", espList);
    }

    private void addDimensionTripOrderTag(final HttpServletRequest request) throws CarAdminException {
        List<JSONObject> dimSettings = new ArrayList<>();
        DimensionIO dimio = ClassFactory.getDimensionIO(si.getSqlFactory());

        SystemPreferences sp = ClassFactory.getSolutionPreferences(si.getSqlFactory());
        int std = sp.readInt(SolutionPreferenceNames.DIMENSION_STANDARD, 0);
        int stdU = sp.readInt(SolutionPreferenceNames.DIMENSION_STANDARD_USER, 0);
        int stdA = sp.readInt(SolutionPreferenceNames.DIMENSION_STANDARD_ASSET, 0);
        Dimension stdDimTrip = dimio.getDimension(std);
        String val = m.t("EconomyDimension.UseCostUnitName");
        if(std > 0 && stdDimTrip != null){
            val = stdDimTrip.getName();
        }
        dimSettings.add( LabelValueFunctionListUtil.getItem(
                        m.t("EconomyDimension.StandardDimensionTrip"),
                        val,
                        "/images/newdesign/edit_104_darkgray.png",
                        "setTripDefaultDimension()"));

        Dimension stdDimUser = dimio.getDimension(stdU);
        String valU = "";
        if(stdDimUser != null){
            valU = stdDimUser.getName();
        }
        dimSettings.add( LabelValueFunctionListUtil.getItem(
                        m.t("EconomyDimension.StandardDimensionUser"),
                        valU,
                        "/images/newdesign/edit_104_darkgray.png",
                        "setUserDefaultDimension()"));

        Dimension stdDimAsset = dimio.getDimension(stdA);
        String valA = "";
        if(stdDimAsset != null){
            valA = stdDimAsset.getName();
        }
        dimSettings.add( LabelValueFunctionListUtil.getItem(
                        m.t("EconomyDimension.StandardDimensionAsset"),
                        valA,
                        "/images/newdesign/edit_104_darkgray.png",
                        "setAssetDefaultDimension()"));

        dimSettings.add(
                LabelValueFunctionListUtil.getItem(
                        m.t("EconomyDimension.Separator"),
                        sp.read(SolutionPreferenceNames.DIMENSION_SEPARATOR, "."),
                        "/images/newdesign/edit_104_darkgray.png",
                        "setDimensionSeparator()"));


        request.setAttribute("dimSettings",dimSettings);
        request.setAttribute("dimHeading",LabelValueFunctionListUtil.getHeaderItem(m.t("Economy.Dimensions")));
    }



    private void addSettingsTag(final HttpServletRequest request,
            final List<JSONObject> objectList,
            final SystemPreferences solutionPreferences,
            final LocationIO lio, ProductFeatures activeFeatures) throws CarAdminException {

        boolean showAll = Rights.hasRight(si, RoleAction.UseApi) && (activeFeatures.isActivated(AssetSetting.PERIODIC_IMPORT) || activeFeatures.isActivated(AssetSetting.ASSET_ECONOMY));

        if(showAll) {
            int defaultLocationId = solutionPreferences.readInt(SolutionPreferenceNames.API_DEFAULT_LOCATION, 1);
            LocationBO defaultLocation = ClassFactory.getLocationIO(si.getSqlFactory()).getLocation(defaultLocationId);
            String lokLabel = m.t("Api.defaultLokasjon");
            String lokValue = (defaultLocation == null) ? "" : defaultLocation.getName();
            objectList.add(
                    LabelValueFunctionListUtil.getItem(
                            lokLabel,
                            lokValue,
                            "/images/newdesign/edit_104_darkgray.png",
                            "getDialog('ApiSettingLocation')"));

            int defaultUserGroupId =
                    Integer.parseInt(solutionPreferences.read("API_defaultUserGroup", "1"));
            UserGroup defaultUserGroup =
                    ClassFactory.getUserGroupIO(si.getSqlFactory())
                            .getUserGroup(defaultUserGroupId);
            String ugLabel = m.t("Api.defaultUserGroup");
            String ugkValue = (defaultUserGroup == null) ? "" : defaultUserGroup.getName();
            objectList.add(
                    LabelValueFunctionListUtil.getItem(
                            ugLabel,
                            ugkValue,
                            "/images/newdesign/edit_104_darkgray.png",
                            "getDialog('ApiSettingUserGroup')"));

            int defaultAssetTypeId =
                    Integer.parseInt(solutionPreferences.read("API_defaultAssetType", "1"));
            AssetType at =
                    ClassFactory.getAssetIO(si.getSqlFactory())
                            .getAssetTypeById(defaultAssetTypeId);
            String typeLabel = m.t("Api.assetTypes");
            String typeValue = (at == null) ? "" : m.t(at.getNameKey());
            objectList.add(
                    LabelValueFunctionListUtil.getItem(
                            typeLabel,
                            typeValue,
                            "/images/newdesign/edit_104_darkgray.png",
                            "getDialog('ApiSettingAssetType')"));
        }

        int defaultInvoiceReceiverId =
                Integer.parseInt(
                        solutionPreferences.read(SolutionPreferenceNames.API_DEFAULT_INVOICE_RECEIVER, "1"));
        InvoiceReceiverBO receiver =
                ClassFactory.getSolutionIO(si.getSqlFactory())
                        .getInvoiceReceiver(defaultInvoiceReceiverId);
        String recevierLabel = m.t("Api.invoiceReceiver");
        String receiverValue = (receiver == null) ? "" : receiver.getName();
        objectList.add(
                LabelValueFunctionListUtil.getItem(
                        recevierLabel,
                        receiverValue,
                        "/images/newdesign/edit_104_darkgray.png",
                        "getDialog('ApiSettingInvoiceReceiver')"));


        if(showAll) {
            int defaultInvoiceDealId =
                    Integer.parseInt(solutionPreferences.read("API_defaultInvoiceDeal", "1"));
            InvoiceDealBO deal =
                    ClassFactory.getInvoiceDealIO(si.getSqlFactory())
                            .getInvoiceDeal(defaultInvoiceDealId);
            String dealLabel = m.t("Api.invoiceDeal");
            String dealValue = (deal == null) ? "" : deal.getName();
            objectList.add(
                    LabelValueFunctionListUtil.getItem(
                            dealLabel,
                            dealValue,
                            "/images/newdesign/edit_104_darkgray.png",
                            "getDialog('ApiSettingInvoiceDeal')"));


            int masterLocationId =
                    Integer.parseInt(
                            solutionPreferences.read(
                                    SolutionPreferenceNames
                                            .IMPORT_LOCATION_SETTINGS_MASTER_LOCATION_ID,
                                    "0"));
            LocationBO location = masterLocationId > 0 ? lio.getLocation(masterLocationId) : null;
            String locLabel = m.t("Api.defaultMasterLocation");
            String locValue = (location == null) ? "" : location.getName();
            objectList.add(LabelValueFunctionListUtil
                                   .getItem(locLabel, locValue, "/images/newdesign/edit_104_darkgray.png", "getDialog('ApiSettingMasterLocation')"));

            //Product
            List<JSONObject> prodList = new ArrayList<>();
            ProductIO pio = ClassFactory.getProductIO(si.getSqlFactory());
            List<ProductBO> assetProds = pio.getProducts();
            if(assetProds == null) {
                assetProds = new ArrayList<>();
            }
            for(ProductBO prod : assetProds) {
                if(prod.isImportDefault()) {
                    prodList.add(LabelValueFunctionListUtil.getItem(prod.getName(),
                                                                    prod.getDescription(),
                                                                    "/images/newdesign/delete.png",
                                                                    "removeProductOnAPI(" + prod.getId() + ")"));
                }
            }


            request.setAttribute("prodHeading",
                                 LabelValueFunctionListUtil
                                         .getHeaderItem(m.t("Api.products"), "", "/images/newdesign/plus.png", "getDialog('ApiSettingsProducts')"));
            request.setAttribute("prodList", prodList);
        }
    }

    private void addRoadMaintenance(final HttpServletRequest request) throws CarAdminException {
        List<JSONObject> colorschemes = new ArrayList<>();
        Map<Integer, String> cschemes = ClassFactory.getRoadMaintenanceIO(si.getSqlFactory()).getColorSchemes();
        int defaultschemeid = ClassFactory.getSolutionPreferences(si.getSqlFactory()).readInt(SolutionPreferenceNames.DEFAULT_COLORSCHEME, 1);


        for(var entry  : cschemes.entrySet()) {
        	int csid = entry.getKey();
            String csname = entry.getValue();
            if(defaultschemeid == csid) {
                csname += " (" + m.t("standard") + ")";
            }
            if(csid == 1) {
                colorschemes.add(LabelValueFunctionListUtil.getItem(
                        csname, " ",
                        "/images/newdesign/edit_104_darkgray.png",
                        "editColorScheme(" + csid + ")"));
            } else {
                colorschemes.add(LabelValueFunctionListUtil.getItem(
                        csname, " ",
                        "/images/newdesign/edit_104_darkgray.png",
                        "editColorScheme(" + csid + ")",
                        "/images/newdesign/delete.png",
                        "removeColorScheme(" + csid + ")"));
            }

        }
        request.setAttribute("roadMaintenanceColorSchemes", colorschemes);
        request.setAttribute(
                "colorSchemeHeading",
                LabelValueFunctionListUtil.getHeaderItem(
                        m.t("Setup.ColorSchemes"),
                        "",
                        "/images/newdesign/plus.png",
                        "getDialog('ColorschemeDialog')"));
    }


    public static void addFuelingTag(final HttpServletRequest request, SolutionSessionInfo si) throws CarAdminException {
        List<JSONObject> fuelImportList = new ArrayList<>();
        FuelIO fio = ClassFactory.getFuelIO(si.getSqlFactory());
        List<JSONObject> fuelImportSuppliers = fio.getFuelImportSuppliers();
        if(fuelImportSuppliers != null && !fuelImportSuppliers.isEmpty()) {
            for(JSONObject f : fuelImportSuppliers) {
                try {
                    fuelImportList.add(
                            LabelValueFunctionListUtil.getItem(
                                    si.getMsg().t("FuelImportFormat." + f.getString("format")),
                                    f.getString("orgname")
                                            + " ("
                                            + f.getString("orgno")
                                            + ")",
                                    "/images/newdesign/delete.png",
                                    "removeImportFormatOnAPI('"
                                            + f.getString("format")
                                            + "')"));
                } catch(JSONException e) {
                    throw new CarAdminException("Noe gikk galt i oppsett av drivstoffting",e);
                }
            }
        }

        request.setAttribute(
                "fuelImportHeading",
                LabelValueFunctionListUtil.getHeaderItem(
                        si.getMsg().t("Api.fuelImport"),
                        "",
                        "/images/newdesign/plus.png",
                        "getDialog('ApiSettingsFuelImport')"));
        request.setAttribute("fuelImportList", fuelImportList);
    }
    
    // @author: Elliot Sveum Torp
    public static void addChargingProvider(final HttpServletRequest request, SolutionSessionInfo si) throws CarAdminException {
        List<JSONObject> chargingProviderList = new ArrayList<>();
        ChargeIO cio = ClassFactory.getChargeIO(si.getSqlFactory());
        List<JSONObject> chargingProviders = cio.getChargingProvidersByStatus(1);

        if(chargingProviders != null && !chargingProviders.isEmpty()) {
            try {
            ChargingProviderLogin cpl = new ChargingProviderLogin();
            JSONArray jsonUserInfo = cpl.getDbUserInfo(si);
            if (jsonUserInfo == null) {
                jsonUserInfo = new JSONArray();
            }

            for(JSONObject c : chargingProviders) {
                String chargingProvider = c.getString("name");
                String image = "";


                for(int i = 0; i < jsonUserInfo.length(); i++) {
                    JSONObject a = jsonUserInfo.getJSONObject(i);
                    String provider = a.getString("provider");
                    String status = a.getString("status");
                    if(Objects.equals(provider, chargingProvider)){
                        if(Objects.equals(status, "registered")) {
                            image = "/images/newdesign/damage/allgood.png";
                        }else if (Objects.equals(status, "wrongPass")) {
                            image = "/images/newdesign/bachelorImages/bachelorXImage.png";
                        }else {
                            image = "/images/newdesign/damage/warn.png";
                        }
                    }
                }

                chargingProviderList.add(
                        LabelValueFunctionListUtil.getItem(
                                chargingProvider,
                                c.getString("username"),
                                image,
                                "editLoginInfo('"
                                        + c.getString("name")
                                        + "')",
                                "/images/newdesign/edit_104_darkgray.png",
                                "editLoginInfo('"
                                        + c.getString("name")
                                        + "')"));
                }
            }catch(Exception e) {
                e.printStackTrace();
            }
        }
        JSONObject headerItem = LabelValueFunctionListUtil.getHeaderItem(
                si.getMsg().t("Api.chargingProviders"),
                "",
                "/images/newdesign/plus.png",
                "getDialog('ApiSettingsChargingProviders')");

        request.setAttribute(
                "chargingProviderHeading",headerItem
                );


        request.setAttribute("chargingProviderList", chargingProviderList);
    }

    public static void addTollTag(final HttpServletRequest request, SolutionSessionInfo si) throws CarAdminException {
        List<JSONObject> tollTagSettingList = new ArrayList<>();
        TollTagIO ttio = ClassFactory.getTollTagIO(si.getSqlFactory());
        List<TollTagSupplier> tollTagSuppliers = ttio.getTollTagSuppliers();
        if(tollTagSuppliers != null && !tollTagSuppliers.isEmpty()) {
            for(TollTagSupplier f : tollTagSuppliers) {
                tollTagSettingList.add(
                        LabelValueFunctionListUtil.getItem(
                                f.getSupplierName()
                                        + " ("
                                        + f.getSupplierOrgNo()
                                        + ")", "",
                                "/images/newdesign/delete.png",
                                "removeTollTagOnAPI('"
                                        + f.getId()
                                        + "')"));
            }
        }
        request.setAttribute(
                "tollTagHeading",
                LabelValueFunctionListUtil.getHeaderItem(
                        si.getMsg().t("Api.TollTagSuppliers"),
                        "",
                        "/images/newdesign/plus.png",
                        "getDialog('ApiSettingsTollTagSupplier')"));
        request.setAttribute("tollTagSettingList", tollTagSettingList);
    }

    @Override
    protected ValueProvider getPageTitleProvider() {
        return new TranslatedStringValueProvider(m, "APISettings");
    }

    @Override
    protected ValueProvider getPageHeaderProvider() {
        return new TranslatedStringValueProvider(m, "APISettings");
    }

    @Override
    protected ValueProvider getViewPathProvider() {
        return new StringValueProvider("/view/admin/apisettings/apisettings.jsp");
    }

}
