package etc.caradmin.mvc.controller.admin.apiSettings;

import etc.beans.user.RoleAction;
import etc.caradmin.io.CarAdminException;
import etc.caradmin.mvc.PageControllerAction;
import etc.caradmin.mvc.PageFragmentController;
import etc.caradmin.mvc.controller.charging.ChargingProviderFormat;
import etc.caradmin.mvc.controller.fueling.FuelImportFormat;
import etc.caradmin.mvc.util.WebUtils;
import etc.caradmin.mvc.util.valueprovider.StringValueProvider;
import etc.caradmin.mvc.util.valueprovider.ValueProvider;
import etc.io.ClassFactory;
import etc.io.expence.ChargeIO;
import etc.io.expence.FuelIO;
import etc.util.crypto.EtcCrypto;
import etc.util.settings.FieldType;
import etc.util.settings.InputField;
import etc.util.settings.SolutionPreferenceNames;
import org.apache.commons.lang3.StringUtils;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;

@PageControllerAction(rights = RoleAction.EditFuelCards)
public class ApiSettingsChargingProviders extends PageFragmentController {
    @Override
    protected void handleRequestImpl(HttpServletRequest request, HttpServletResponse response) {
        String edit = WebUtils.getStringOrDefault(request, requestInformation, si, "edit");
        String format = WebUtils.getStringOrDefault(request, requestInformation, si, "format");
        String username = WebUtils.getStringOrDefault(request, requestInformation, si, "username");
        String password = WebUtils.getStringOrDefault(request, requestInformation, si, "password");

        ChargeIO cio = ClassFactory.getChargeIO(si.getSqlFactory());
        try {
            if(StringUtils.isNotBlank(format)) { // && supplierId > 0
                ChargingProviderFormat cpf = ChargingProviderFormat.valueOf(format.toUpperCase());

                EtcCrypto crypto = new EtcCrypto();

                cio.updateChargingProviderUserInfo(username, crypto.encrypt(password.getBytes()), cpf.name(), 1);

                redirect(request, response, "/" + solutionUrl + "/x/SolutionSetup/");

            } else {
                List<InputField> fields = new ArrayList<>();
                List<String> atNames = new ArrayList<>();
                List<String> atValues = new ArrayList<>();
                String solFormats = ClassFactory.getSolutionPreferences(si.getSqlFactory()).read(SolutionPreferenceNames.CHARGING_PROVIDER_FORMAT, "EVINY,EON");
                List<ChargingProviderFormat> formats = new ArrayList<>();
                if(StringUtils.isNotBlank(solFormats)) {
                    String[] formArr = solFormats.split(",");
                    for(String s : formArr) {
                        try {
                            formats.add(ChargingProviderFormat.valueOf(s));
                        } catch(IllegalArgumentException e) {
                            //ignored
                        }
                    }
                }
                Collections.sort(formats);
                if(StringUtils.isNotBlank(edit)) {
                    atNames.add(si.getMsg().t("ChargingProvider." + edit.toUpperCase()));
                    atValues.add(edit);
                }else {
                    for(ChargingProviderFormat fif : formats) {
                        atNames.add(si.getMsg().t("ChargingProvider." + fif.name()));
                        atValues.add(fif.name());
                    }
                }


                InputField atField = InputField
                        .create("chargingProvider", "", FieldType.ChoiceSettingNoTranslate, null, null, 1, atNames.toArray(), atValues.toArray());
                fields.add(atField);
                String[] values = new String[1];
                values[0] = "";
                //InputField f = InputField.create("provider", 0, FieldType.DatabaseCollectionSetting, null, null, 1, new String[] {"SupplierSearch"}, values);
                //if(si.getLocation() != null) {
                //    HashMap<String, String> adds = new HashMap<>();
                //    adds.put("searchType", "provider");
                //    f.setAdditionalParameters(adds);
                //}
                //f.setExtraFunction("addChargingProvider");
                //f.setExtraFunctionIcon("/images/simpleicons/plus_gray.png");

                InputField u = InputField.create("username", 0, FieldType.DatabaseCollectionSetting, null, null, 1, null, null);
                InputField p = InputField.create("password", "", FieldType.Password, null, null, 0, null, null);

                //fields.add(f);
                fields.add(u);
                fields.add(p);

                request.setAttribute("fields", fields);
            }
        } catch(CarAdminException | IOException e) {
            getLogger().error("Noe gikk feil i settings for chargingproviders", e);
        }
    }

    @Override
    protected ValueProvider getViewPathProvider() {
        return new StringValueProvider("/view/admin/apisettings/editApiChargingProviders.jsp");
    }
}
