package etc.caradmin.mvc.controller.charging;


import etc.beans.user.RoleAction;
import etc.beans.user.SolutionSessionInfo;
import etc.caradmin.beans.table.ReportParametersModern;
import etc.caradmin.beans.table.ReportPeriodFilter;
import etc.caradmin.mvc.PageControllerAction;
import etc.caradmin.mvc.controller.reports.ReportPageController;
import etc.caradmin.mvc.util.valueprovider.StringValueProvider;
import etc.caradmin.mvc.util.valueprovider.TranslatedStringValueProvider;
import etc.caradmin.mvc.util.valueprovider.ValueProvider;
import etc.util.DateTime;
import etc.util.json.JSONArray;
import etc.util.report.DataGridUtil;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.Calendar;



/**
 * Controller for oversikt over drivstoffyllinger
 * @author Sindre Emil Vikre
 */
@PageControllerAction(rights= RoleAction.ViewFuelbook, icon="/images/menuicons/fueling_history_black-104_02.png")
public class ChargingHistory extends ReportPageController {

    public ChargingHistory() {
        super("chargingParameters");
    }

    public ChargingHistory(SolutionSessionInfo si, String reportParameterFilterName) {
        super(reportParameterFilterName);
        this.si = si;
        this.m = si.getMsg();
    }

    /**
     *  setupDefaultReportParameters returnerer default filter til rapporten.
     *
     * @return - returnerer filter til rapporten.
     */
    @Override
    protected ReportParametersModern setupDefaultReportParameters() {
        ReportParametersModern reportParas = new ReportParametersModern();
        reportParas.setOrderByColumn(1);
        if(si.getLocation() == null){
            reportParas.setFilterLocation(ReportParametersModern.FilterLocation.ALL);
        }
        else{
            reportParas.setFilterLocation(ReportParametersModern.FilterLocation.SINGLE);
        }
        reportParas.setHasPeriodFilter(true);
        reportParas.setPeriodStart(DateTime.now().setDayOfMonth(1));
        reportParas.setPeriodStop(DateTime.now().setDayOfMonth(1).timeShift(1,Calendar.MONTH).timeShift(-1, Calendar.DAY_OF_YEAR).setHour(23).setMinute(59).setSecond(59) );
        reportParas.setMainFilterMode(ReportParametersModern.MainFilterMode.PERIOD);
        reportParas.setPeriodMode(ReportPeriodFilter.PeriodMode.Month);


        return reportParas;
    }

    /**
     *
     * handleRequestImpl blir kjrt nr en bruker pener ChargingHistory siden i CarAdmin.
     *
     * @param request  foresprsel
     * @param response respons
     */
    @Override
    protected void handleRequestImpl(HttpServletRequest request, HttpServletResponse response) {


        //Setter parametere til rapport filteret.
        ReportParametersModern reportParas = getReportParameters(request);
        reportParas.setFilterTitle(m.t("Chargings"));
        reportParas.setFilterName(reportParameterName);
        reportParas.setHasPeriodFilter(true);
        reportParas.setElementsPerPage(20);
        reportParas.setFilterLocation(ReportParametersModern.FilterLocation.ALL);


        //Bestemmer hva rapporten p nettsiden skal inneholde.
        JSONArray mainLo = new JSONArray();
        mainLo.put(DataGridUtil.getColumnDescriptor("id", "id", true));
        mainLo.put(DataGridUtil.getColumnDescriptor("charging.brand", m.t("Charger.Brand"), false));
        mainLo.put(DataGridUtil.getColumnDescriptor("charging.chargerId", m.t("Charger.ChargerId"), false));
        mainLo.put(DataGridUtil.getColumnDescriptor("charging.date", m.t("Charger.Date"), false));
        mainLo.put(DataGridUtil.getColumnDescriptor("charging.price", m.t("Charger.Price"), false));
        mainLo.put(DataGridUtil.getColumnDescriptor("charging.kwh", m.t("Charger.Kwh"), false));

        mainLo.put(DataGridUtil.getColumnDescriptor("id",m.t("Action"),false,"reportDetailsFormatter").put("headerFormatter","reportActionHeaderFormatter").put("actionfield","opendetails").put("isactioncol",true));

        //Sender layout og filter tilbake til jsp filen.
        request.setAttribute("mainLayout", mainLo);
        request.setAttribute(reportParameterName, reportParas);

    }


    @Override
    protected ValueProvider getPageTitleProvider() {
        return new TranslatedStringValueProvider(m, "Chargings");

    }

    @Override
    protected ValueProvider getPageHeaderProvider() {
        return new TranslatedStringValueProvider(m, "Chargings");
    }

    @Override
    protected ValueProvider getViewPathProvider() {
        return new StringValueProvider("/view/charging/chargingHistory.jsp"); //jsp filen som gir utseende til denne nettsiden.
    }


}
