package etc.caradmin.mvc.controller.charging;

import etc.beans.customer.CustomerBO;
import etc.beans.user.RoleAction;
import etc.caradmin.io.CarAdminException;
import etc.caradmin.mvc.PageControllerAction;
import etc.caradmin.mvc.PageFragmentController;
import etc.caradmin.mvc.controller.fueling.FuelCard;
import etc.caradmin.mvc.util.WebUtils;
import etc.caradmin.mvc.util.valueprovider.StringValueProvider;
import etc.caradmin.mvc.util.valueprovider.ValueProvider;
import etc.io.ClassFactory;
import etc.io.customer.CustomerIO;
import etc.io.expence.ChargeIO;
import etc.io.expence.FuelIO;
import etc.util.schema.InputSchemaUtils;
import etc.util.settings.FieldType;
import etc.util.settings.InputField;
import lombok.extern.log4j.Log4j2;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;


@Log4j2
@PageControllerAction(rights= RoleAction.EditFuelCards)
public class ChargingProviderRegisterDialog extends PageFragmentController {
    @Override
    protected void handleRequestImpl(final HttpServletRequest request, final HttpServletResponse response) /*throws IOException*/ {
        int id = WebUtils.getIntOrDefault(request, requestInformation, si, "fuelCardId");
        boolean fromAssetSettings = WebUtils.getBoolOrDefault(request, requestInformation, si, "fromAssetSettings");
        request.setAttribute("fromAssetSettings",fromAssetSettings);
        FuelCard fc = new FuelCard();

        //Henter valgte roderader
        String kjoretoy = WebUtils.getStringOrDefault(request, null, "kjoretoy");
        String rfidbrikker = WebUtils.getStringOrDefault(request, null, "rfidbrikker");
        request.setAttribute("kjoretoy", kjoretoy);
        request.setAttribute("rfidbrikker", rfidbrikker);

        Map<String, String> RFIDandCar;

        if(id > 0){
            //redigering
            FuelIO fio = ClassFactory.getFuelIO(si.getSqlFactory());
            ChargeIO cio = ClassFactory.getChargeIO(si.getSqlFactory());
            try {
                //fc = fio.getFuelCardById(id);
                RFIDandCar = cio.getChargingTagAndCarIdById(id);

                if(RFIDandCar != null){
                    request.setAttribute("rfidbrikker", RFIDandCar.get("rfidNumber"));
                    request.setAttribute("kjoretoy",RFIDandCar.get("carID"));
                }
            } catch (CarAdminException e) {
                log.error("Klarte ikke hente ut drivstoffkort",e);
            }
        }

        List<InputField> fields = null;
        if (fc != null) {
            fields = InputSchemaUtils.getFields(fc, null);
        }
        CustomerIO cuio = ClassFactory.getCustomerIO(si.getSqlFactory());

        if (fields != null) {
            for(InputField f : fields){
                if(fromAssetSettings){
                    if(f.getName().equalsIgnoreCase("cardNumber")){
                        f.setType(FieldType.StringSettingReadOnly);
                    }
                }
                if(f.getName().equalsIgnoreCase("description")){
                    f.setPlaceholder(m.t("FuelCard.descriptionHelp"));
                    f.setArguments(new String[]{"optionShowLabelInline"});
                }
                if(f.getName().equalsIgnoreCase("oilCompany")){
                    String val = "";
                    if(fc.getOilCompany() > 0){
                        try {
                            CustomerBO sup = cuio.getCustomerByID(fc.getOilCompany());
                            if(sup != null){
                                val = sup.getOrgName();
                            }
                        } catch (CarAdminException e) {
                            getLogger().error("klarte ikke hente leverand�r",e);
                        }
                    }
                    f.setArguments(new String[]{"SupplierSearch"});
                    List<String> pValues = new ArrayList<>();
                    pValues.add(Integer.toString(fc.getOilCompany()));
                    f.setValues(pValues.toArray());

                    if (si.getLocation() != null) {
                        HashMap<String, String> adds = new HashMap<>();
                        adds.put("searchType", "supplier");
                        f.setAdditionalParameters(adds);
                    }
                    if(!fromAssetSettings) {
                        f.setExtraFunction("addFuelCardSupplier");
                        f.setExtraFunctionIcon("/images/newdesign/Plus_64px_gray.png");
                    }
                    f.setValue(val);
                    f.setOrder(1);

                }
            }
        }

        request.setAttribute("fields",fields);
    }

    @Override
    protected ValueProvider getViewPathProvider() {
        return new StringValueProvider("/view/charging/ChargingProviderRegisterDialog.jsp");
    }
}
