package etc.caradmin.mvc.controller.charging;


import etc.ajax.chargingTagProviderParsing.ChargingTagProviderParser;
import etc.beans.gdpr.GdprPrivacyContext;
import etc.beans.user.RoleAction;
import etc.caradmin.io.CarAdminException;
import etc.caradmin.mvc.PageControllerAction;
import etc.caradmin.mvc.PageFragmentController;
import etc.caradmin.mvc.util.WebUtils;
import etc.caradmin.mvc.util.valueprovider.StringValueProvider;
import etc.caradmin.mvc.util.valueprovider.ValueProvider;
import etc.io.ClassFactory;
import etc.io.expence.ChargeIO;
import etc.io.expence.FuelIO;
import etc.io.logging.expence.LoggingChargingIO;
import etc.io.logging.expence.LoggingFuelIO;
import etc.util.json.JSONException;
import etc.util.json.JSONObject;
import etc.util.settings.FieldType;
import etc.util.settings.InputField;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

/**
 * Dialog for � knytte kjr�et�y eller brukere til et drivstoffkort
 *  @author Rihards D. U.
 */
@PageControllerAction(rights= RoleAction.EditFuelCards)
public class ChargingTagBindingDialog extends PageFragmentController {
    @Override
    protected void handleRequestImpl(HttpServletRequest request, HttpServletResponse response) {
        int tagId = WebUtils.getIntOrDefault(request, requestInformation, si, "chargingTagId");
        boolean bindAsset = WebUtils.getBoolOrDefault(request, requestInformation, si, "bindAsset");
        boolean bindUser = WebUtils.getBoolOrDefault(request, requestInformation, si, "bindUser");
        boolean saveUser = WebUtils.getBoolOrDefault(request, requestInformation, si, "saveUser");
        boolean saveAsset = WebUtils.getBoolOrDefault(request, requestInformation, si, "saveAsset");
        boolean save = WebUtils.getBoolOrDefault(request, requestInformation, si, "save");
        boolean deleteUser = WebUtils.getBoolOrDefault(request, requestInformation, si, "deleteUserBinding");
        boolean deleteAsset = WebUtils.getBoolOrDefault(request, requestInformation, si, "deleteAssetBinding");
        boolean deleteProvider = WebUtils.getBoolOrDefault(request, requestInformation, si, "deleteProvider");


        if (tagId > 0) {
            ChargeIO cio = ClassFactory.getChargeIO(si.getSqlFactory());
            LoggingChargingIO loggingChargingIO = ClassFactory.getLoggingChargingIO(si.getSqlFactory());
            if (save) {
                try {

                    JSONObject ret = new JSONObject();
                    if (saveUser) {//lagrer knytning til bruker
                        int userId = WebUtils.getIntOrDefault(request, requestInformation, si, "userId");
                        if (userId > 0) {
                            loggingChargingIO.addChargingTagBinding(tagId, 0, userId, si, GdprPrivacyContext.ChargingTagSave);
                        } else {
                            ret.put("error", true);
                            ret.put("customError", "bindUser");
                            ret.put("errorString", si.getMsg().t("ChargingTag.UserError"));
                        }
                    } else if (saveAsset) {//lagrer knytning til kj�ret�y
                        int assetId = WebUtils.getIntOrDefault(request, requestInformation, si, "assetId");
                        if (assetId > 0) {
                            cio.addChargingTagBinding(tagId, assetId, 0);
                        } else {
                            ret.put("error", true);
                            ret.put("customError", "bindAsset");
                            ret.put("errorString", si.getMsg().t("ChargingTag.AssetError"));
                        }
                    }

                    ret.write(response.getWriter());
                    response.getWriter().flush();
                } catch (CarAdminException | JSONException | IOException e) {
                    getLogger().error("Noe gikk galt i dialog for � knytte ladebrikke", e);
                    response.setStatus(HttpServletResponse.SC_INTERNAL_SERVER_ERROR);
                }
            } else if (deleteUser) {//Fjerner brukerknytting
                int userId = WebUtils.getIntOrDefault(request, requestInformation, si, "deleteUserId");
                if (userId > 0) {
                    removeBinding(response, tagId, loggingChargingIO, userId, 0);
                }
            } else if (deleteAsset) {//Fjerner kj�ret�yknytting
                int assetId = WebUtils.getIntOrDefault(request, requestInformation, si, "deleteAssetId");
                if(assetId > 0) {
                    removeBinding(response, tagId, loggingChargingIO, 0, assetId);
                }
            } else if(deleteProvider){
                int chargingTagId = WebUtils.getIntOrDefault(request, requestInformation, si, "chargingTagId");
                int deleteProviderId = WebUtils.getIntOrDefault(request, requestInformation, si, "deleteProviderId");
                if (chargingTagId > 0 && deleteProviderId > 0){
                    removeProviderBinding(response, chargingTagId, loggingChargingIO, cio, deleteProviderId);
                }
            } else { //viser knytte-dialog
                List<InputField> fields = new ArrayList<>();
                String[] values = new String[1];
                values[0] = "";


                if (bindAsset) {
                    try {
                        List<JSONObject> binds = cio.getBindingsForTag(tagId);
                        if(binds != null){
                            for(JSONObject j : binds){
                                if(j.has("carid") && j.has("assetno")){
                                    request.setAttribute("hasBoundAsset",true);
                                    break;
                                }
                            }
                        }
                    } catch (CarAdminException e) {
                        getLogger().error("Klarte ikke hent ut bindinger for brikke",e);
                    }
                    InputField assetSearch = InputField.create("bindAsset", "", FieldType.DatabaseCollectionSetting, null, null, 1, new String[]{"VehicleSearchStore"}, values);
                    fields.add(assetSearch);
                    HashMap<String, String> adds = new HashMap<>();
                    adds.put("checkSetting", "ChargingTag");
                    assetSearch.setAdditionalParameters(adds);
                    request.setAttribute("bindAsset", true);
                } else if (bindUser) {
                    InputField userSearch = InputField.create("bindUser", "", FieldType.DatabaseCollectionSetting, null, null, 1, new String[]{"UserSearchStore"}, values);
                    fields.add(userSearch);
                    request.setAttribute("bindUser", true);
                }
                request.setAttribute("chargingTagId", tagId);
                request.setAttribute("fields", fields);
            }
        }
    }


    private void removeBinding(HttpServletResponse response, int tagId, LoggingChargingIO cio, int userId, int assetId) {
        try {
            cio.removeChargingTagBinding(tagId, assetId, userId, si, GdprPrivacyContext.ChargingTagRemove);
            JSONObject ret = new JSONObject();
            ret.put("deletedok", true);
            ret.write(response.getWriter());
            response.getWriter().flush();
        } catch (CarAdminException | JSONException | IOException e) {
            getLogger().error("Noe gikk galt ved fjerning av bruker-knytning p� ladebrikke", e);
        }
    }

    private void removeProviderBinding(HttpServletResponse response, int tagId, LoggingChargingIO lcio, ChargeIO cio, int providerId) {
        try {
            //cio.removeChargingProviderBinding(tagId, providerId, si, GdprPrivacyContext.ChargingProviderRemove);
            ChargingTagProviderParser parser = new ChargingTagProviderParser();

            if(providerId == 1){
                ChargingProvider cp = cio.getChargingProviderById(providerId);
                String token = parser.bilkraftLogin(cp.getUserName(), cp.getPassword());
                String rfidnummer = cio.getChargingTagById(tagId).getRfidNumber();
                if (parser.bilkraftDelete(rfidnummer,token).equals("success")) {
                    lcio.removeChargingProviderBinding(tagId, providerId, si, GdprPrivacyContext.ChargingProviderRemove);
                    JSONObject ret = new JSONObject();
                    ret.put("deletedok", true);
                    ret.write(response.getWriter());
                    response.getWriter().flush();
                }
            }

            if(providerId == 2){
                ChargingProvider cp = cio.getChargingProviderById(providerId);
                String token = parser.eonLogin(cp.getUserName(), cp.getPassword());
                String rfidnummer = cio.getChargingTagById(tagId).getRfidNumber();
                if (parser.eonDelete(rfidnummer,token).equals("success")) {
                    lcio.removeChargingProviderBinding(tagId, providerId, si, GdprPrivacyContext.ChargingProviderRemove);
                    JSONObject ret = new JSONObject();
                    ret.put("deletedok", true);
                    ret.write(response.getWriter());
                    response.getWriter().flush();
                }
            }

        } catch(CarAdminException | JSONException | IOException e) {
            getLogger().error("Noe gikk galt ved fjerning av ladeselskap p� ladebrikken", e);
        }
    }

    @Override
    protected ValueProvider getViewPathProvider() {
        return new StringValueProvider("/view/charging/chargingTagBindingDialog.jsp");
    }
}
