package etc.caradmin.mvc.controller.charging;

import etc.beans.gdpr.GdprPrivacyContext;
import etc.beans.user.RoleAction;
import etc.caradmin.asset.AssetSetting;
import etc.caradmin.io.CarAdminException;
import etc.caradmin.mvc.PageControllerAction;
import etc.caradmin.mvc.function.DialogFunctionItem;
import etc.caradmin.mvc.function.FunctionItem;
import etc.caradmin.mvc.util.ReportDetailController;
import etc.caradmin.mvc.util.WebUtils;
import etc.io.ClassFactory;
import etc.io.customer.CustomerIO;
import etc.io.expence.ChargeIO;
import etc.io.logging.user.LoggingUserIO;
import etc.util.json.JSONException;
import etc.util.json.JSONObject;
import etc.util.report.ReportDetail;
import etc.util.report.detail.FunctionReportDetail;
import etc.util.report.detail.LabelFunctionReportDetail;
import etc.util.report.detail.LabelReportDetail;
import org.jetbrains.annotations.NotNull;

import javax.servlet.http.HttpServletRequest;
import java.util.ArrayList;
import java.util.List;

/**
 * Detaljkontroller for drivstoffkort
 * @author Rihards D. U.
 */
@PageControllerAction(rights= RoleAction.EditFuelCards)
public class ChargingTagDetails extends ReportDetailController {
    @NotNull
    @Override
    protected List<ReportDetail> getReportDetails(HttpServletRequest request) throws CarAdminException {
        int tagId = WebUtils.getIntOrDefault(request, requestInformation, si, "chargingTagId");
        String kjoretoy = WebUtils.getStringOrDefault(request, requestInformation, si, "kjoretoy");
        String rfidbrikker = WebUtils.getStringOrDefault(request, requestInformation, si, "rfidbrikker");
        if(tagId > 0){
            ChargeIO cio = ClassFactory.getChargeIO(si.getSqlFactory());
            LoggingUserIO uio = ClassFactory.getLoggingUserIO(si.getSqlFactory());
            ChargingTag ct = cio.getChargingTagById(tagId);
            CustomerIO cuio = ClassFactory.getCustomerIO(si.getSqlFactory());
            if(ct != null){
                List<ReportDetail> dets = new ArrayList<>();
                LabelReportDetail det = new LabelReportDetail();
                det.setHeading(m.t("ChargingTag.Info"));
                det.addData(m.t("ChargingTag.rfidNumber"), ct.getRfidNumber());

                det.addData(m.t("ChargingTag.created"), ct.getCreated() != null ? ct.getCreated().getFormatedDateDate(m) : "");
                if(ct.getCreatedBy() > 0){
                    String name = uio.getNameOfUser(ct.getCreatedBy(), si, GdprPrivacyContext.ChargingTagCreatedBy);
                    det.addData(m.t("ChargingTag.createdBy"), name);
                }
                det.setDetailSize(3);
                dets.add(det);

                // User and asset
                List<JSONObject> bindings = cio.getBindingsForTag(tagId);
                LabelFunctionReportDetail lfrd = new LabelFunctionReportDetail();
                lfrd.setHeading(m.t("ChargingTag.Assets"));
                LabelFunctionReportDetail lfrd2 = new LabelFunctionReportDetail();
                lfrd2.setHeading(m.t("ChargingTag.Users"));

                boolean assetsAdded = false;
                boolean usersAdded = false;
                try {
                    if(bindings != null && !bindings.isEmpty()){
                        for(JSONObject j : bindings){
                            if(j.has("carid") && j.has("assetno")){
                                lfrd.addData("",j.getString("carid"),"/images/newdesign/delete.png","removeAssetBinding("+tagId+","+j.getInt("assetno")+",'"+j.getString("carid")+"')");
                                assetsAdded = true;
                            }
                            if(j.has("userid") && j.has("username")){
                                lfrd2.addData("",j.getString("username"),"/images/newdesign/delete.png","removeUserBinding("+tagId+","+j.getInt("userid")+",'"+j.getString("username")+"')");
                                usersAdded = true;
                            }
                        }
                    }
                } catch (JSONException e) {
                    getLogger().error("Noe gikk galt i utlisting av knyttingner for ladebrikke",e);
                }
                if(assetsAdded) {
                    dets.add(lfrd);
                }
                if(usersAdded){
                    dets.add(lfrd2);
                }

                // Charging provider
                List<JSONObject> providerBindings = cio.getBindingsForProvider(tagId);
                LabelFunctionReportDetail lfrd3 = new LabelFunctionReportDetail();
                lfrd3.setHeading(m.t("ChargingProvider.BindChargingProviders"));

                boolean providerAdded = false;
                try {
                    if(providerBindings != null && !providerBindings.isEmpty()){
                        for(JSONObject j : providerBindings){
                            if(j.has("name") && j.has("providerid") && j.has("status")){
                                if (j.getInt("status") == 1) {
                                    lfrd3.addData("", j.getString("name"), "/images/newdesign/damage/allgood.png", "", "/images/newdesign/delete.png", "removeProviderBinding("+tagId+", "+j.getInt("providerid")+", '"+j.getString("name")+"')");
                                }else if (j.getInt("status") == 2){
                                    lfrd3.addData("", j.getString("name"), "/images/newdesign/damage/warn.png", "", "/images/newdesign/delete.png", "removeProviderBinding("+tagId+", "+j.getInt("providerid")+", '"+j.getString("name")+"')");
                                }else{
                                    lfrd3.addData("", j.getString("name"), "", "", "/images/newdesign/delete.png", "removeProviderBinding("+tagId+", "+j.getInt("providerid")+", '"+j.getString("name")+"')");
                                }
                                providerAdded = true;
                            }
                        }
                    }
                }   catch (JSONException e) {
                    getLogger().error("Noe gikk galt i utlisting av knyttingner for ladebrikke", e);
                }
                if(providerAdded){
                    dets.add(lfrd3);
                }

                FunctionReportDetail r4 = new FunctionReportDetail();
                List<FunctionItem> filtered = new ArrayList<>();
                JSONObject idParam = new JSONObject();
                idParam.put("chargingTagId", tagId);
                filtered.add(new DialogFunctionItem("editChargingTag", m.t("ChargingTag.Edit"), "/images/newdesign/edit_104_darkgray.png", "ChargingTagDialog", RoleAction.EditFuelCards, AssetSetting.FUEL_CARD, idParam, null, ""));

                //Sjekker om brikken allerede har en tillknytet bil
                if(cio.getChargingTagAndCarIdById(tagId) == null) {
                    JSONObject carParam = new JSONObject();
                    carParam.put("chargingTagId", tagId);
                    carParam.put("bindAsset", true);
                    filtered.add(new DialogFunctionItem("bindChargingTag",
                                                        m.t("ChargingTag.BindAsset"),
                                                        "/images/newdesign/Link_darkgray.png",
                                                        "ChargingTagBindingDialog",
                                                        RoleAction.EditFuelCards,
                                                        AssetSetting.FUEL_CARD,
                                                        carParam,
                                                        null,
                                                        ""));
                }

                JSONObject userParam = new JSONObject();
                userParam.put("chargingTagId", tagId);
                userParam.put("bindUser",true);
                filtered.add(new DialogFunctionItem("bindChargingTag", m.t("ChargingTag.BindUser"), "/images/newdesign/Link_darkgray.png", "ChargingTagBindingDialog", RoleAction.EditFuelCards, AssetSetting.FUEL_CARD, userParam, null, ""));
                JSONObject chargingTagParam = new JSONObject();

                chargingTagParam.put("fuelCardId", tagId);
                chargingTagParam.put("kjoretoy", kjoretoy);
                chargingTagParam.put("rfidbrikker", rfidbrikker);
                filtered.add(new DialogFunctionItem("bindChargingProvider", "Ladebrikke reg.", "/images/newdesign/bachelorImages/ladestasjon.png", "ChargingProviderRegisterDialog", RoleAction.EditFuelCards, AssetSetting.FUEL_CARD, chargingTagParam, null, ""));

                r4.setFunctions(filtered);
                dets.add(r4);
                r4.setDetailSize(3);
                return dets;
            }
        }

        return null;
    }
}
