package etc.caradmin.mvc.controller.charging;


import etc.beans.user.RoleAction;
import etc.caradmin.io.CarAdminException;
import etc.caradmin.mvc.PageControllerAction;
import etc.caradmin.mvc.PageFragmentController;
import etc.caradmin.mvc.util.WebUtils;
import etc.caradmin.mvc.util.valueprovider.StringValueProvider;
import etc.caradmin.mvc.util.valueprovider.ValueProvider;
import etc.io.ClassFactory;
import etc.io.customer.CustomerIO;
import etc.io.expence.ChargeIO;
import etc.util.schema.InputSchemaUtils;
import etc.util.settings.FieldType;
import etc.util.settings.InputField;
import lombok.extern.log4j.Log4j2;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.List;

/**
 * dialog for � opprette eller redigere drivstoffkort
 * @author Rihards D. U.
 */
@Log4j2
@PageControllerAction(rights= RoleAction.EditFuelCards)
public class ChargingTagDialog extends PageFragmentController {
    @Override
    protected void handleRequestImpl(HttpServletRequest request, HttpServletResponse response) {
        int id = WebUtils.getIntOrDefault(request, requestInformation, si, "chargingTagId");
        boolean fromAssetSettings = WebUtils.getBoolOrDefault(request, requestInformation, si, "fromAssetSettings");
        request.setAttribute("fromAssetSettings",fromAssetSettings);
        ChargingTag ct = new ChargingTag();
        if(id > 0){
            //redigering
            ChargeIO fio = ClassFactory.getChargeIO(si.getSqlFactory());
            try {
                ct = fio.getChargingTagById(id);
                if(ct != null && ct.getId() > 0){
                    request.setAttribute("chargingTagId",ct.getId());
                }else{
                    request.setAttribute("tagError",true);
                }
            } catch (CarAdminException e) {
                log.error("Klarte ikke hente ut ladebrikke",e);
            }
        }

        List<InputField> fields = null;
        if (ct != null) {
            fields = InputSchemaUtils.getFields(ct, null);
        }
        CustomerIO cuio = ClassFactory.getCustomerIO(si.getSqlFactory());

        if (fields != null) {
            for(InputField f : fields){
                if(fromAssetSettings){
                    if(f.getName().equalsIgnoreCase("rfidNumber")){
                        f.setType(FieldType.StringSettingReadOnly);
                    }
                }
                if(f.getName().equalsIgnoreCase("description")){
                    f.setPlaceholder(m.t("ChargingTag.descriptionHelp"));
                    f.setArguments(new String[]{"optionShowLabelInline"});
                }
            }
        }

        request.setAttribute("fields",fields);
    }

    @Override
    protected ValueProvider getViewPathProvider() {
        return new StringValueProvider("/view/charging/chargingTagDialog.jsp");
    }
}
