package etc.caradmin.mvc.controller.charging;

import etc.beans.table.ReportParameterUtilModern;
import etc.beans.user.Rights;
import etc.beans.user.RoleAction;
import etc.beans.user.SolutionSessionInfo;
import etc.caradmin.asset.AssetSetting;
import etc.caradmin.beans.table.ColumnCheckbox;
import etc.caradmin.beans.table.ModernReportFilter;
import etc.caradmin.beans.table.ReportParametersModern;
import etc.caradmin.io.CarAdminException;
import etc.caradmin.mvc.PageControllerAction;
import etc.caradmin.mvc.controller.reports.ReportPageController;
import etc.caradmin.mvc.function.DialogFunctionItem;
import etc.caradmin.mvc.function.FunctionItem;
import etc.caradmin.mvc.function.LocalJavascriptFunctionItem;
import etc.caradmin.mvc.util.valueprovider.StringValueProvider;
import etc.caradmin.mvc.util.valueprovider.TranslatedStringValueProvider;
import etc.caradmin.mvc.util.valueprovider.ValueProvider;
import etc.util.json.JSONArray;
import etc.util.report.DataGridUtil;
import org.jetbrains.annotations.NotNull;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.ArrayList;
import java.util.List;

/**
 * Oversikt over ladebrikker
 * @author Rihards D. U.
 */
@PageControllerAction(rights= RoleAction.EditFuelCards)
public class ChargingTagOverview extends ReportPageController {

    public ChargingTagOverview(String reportParameterName) {
        super(reportParameterName);
    }

    public ChargingTagOverview() {super("chargingTagParameters");}

    public ChargingTagOverview(SolutionSessionInfo si, String reportParameterFilterName) {
        super(reportParameterFilterName);
        this.si = si;
        this.m = si.getMsg();
    }

    @Override
    protected ReportParametersModern setupDefaultReportParameters() {
        ReportParametersModern reportParas = new ReportParametersModern();
        reportParas.setOrderByColumn(1);
        if(si.getLocation() == null){
            reportParas.setFilterLocation(ReportParametersModern.FilterLocation.ALL);
        }
        else{
            reportParas.setFilterLocation(ReportParametersModern.FilterLocation.SINGLE);
        }

        return reportParas;
    }

    @Override
    protected void handleRequestImpl(HttpServletRequest request, HttpServletResponse response) {
        ReportParametersModern reportParas = getReportParameters(request);
        reportParas.setFilterTitle(m.t("ChargingTag.ChargingTags"));
        reportParas.setFilterName(reportParameterName);

        ModernReportFilter search = new ModernReportFilter();
        search.setHtmlId("chargingTagSearch");
        search.setMainFilter(true);
        search.setTag(ModernReportFilter.FilterTag.SEARCHFIELD);
        reportParas.setMainFilter(search);
        reportParas.setSearchPlaceholder(m.t("ChargingTag.Search"));
        reportParas.setMainFilterMode(ReportParametersModern.MainFilterMode.CUSTOM);

        ModernReportFilter checkboxfilter = reportParas.getFilterByHtmlId("columnCheckboxFilter");
        if (checkboxfilter == null) {
            checkboxfilter = getCheckBoxFilter(reportParas);
        }

        JSONArray mainLo = new JSONArray();
        mainLo.put(DataGridUtil.getColumnDescriptor("id", "id", true));
        mainLo.put(DataGridUtil.getColumnDescriptor("rfidNumber", m.t("ChargingTag.rfidNumber"), false));
        mainLo.put(DataGridUtil.getColumnDescriptor("rfidNumber", m.t("ChargingTag.rfidNumber"), true));
        mainLo.put(DataGridUtil.getColumnDescriptor("assets", m.t("ChargingTag.bindAsset"), false));
        mainLo.put(DataGridUtil.getColumnDescriptor("assets", m.t("ChargingTag.bindAsset"), true));
        mainLo.put(DataGridUtil.getColumnDescriptor("description", m.t("ChargingTag.description"), false));
        mainLo.put(DataGridUtil.getColumnDescriptor("providers", m.t("ChargingTag.chargeCompany"), false));
        mainLo.put(DataGridUtil.getColumnDescriptor("users", m.t("ChargingTag.bindUser"), false));
        mainLo.put(DataGridUtil.getColumnDescriptor("createdby", m.t("ChargingTag.createdBy"), false));
        mainLo.put(DataGridUtil.getColumnDescriptor("created", m.t("ChargingTag.created"), false));
        mainLo.put(DataGridUtil.getColumnDescriptor("id", m.t("Action"), false,"reportDetailsFormatter").put("headerFormatter","reportActionHeaderFormatter").put("headerColspan",1) );
        mainLo.put(DataGridUtil.getColumnDescriptor("id","", false, "reportCheckbox").put("headerFormatter", "reportHeaderCheckbox"));


        request.setAttribute("mainLayout",mainLo);
        request.setAttribute("chargingTagParameters",reportParas);


        showHideFields(mainLo, checkboxfilter.getColumnCheckBoxes());
        request.setAttribute("mainLo", mainLo);
        List<FunctionItem> func = new ArrayList<>();
        List<FunctionItem> hidd = new ArrayList<>();
        try {
            if(Rights.hasRight(si, RoleAction.EditFuelCards)) {
                // Registrer nye rfid brikker
                func.add(new DialogFunctionItem("newChargingTag", m.t("ChargingTag.NewChargingTag"), "/images/newdesign/Plus-2-Math-104.png", "ChargingTagDialog", RoleAction.EditFuelCards, AssetSetting.FUEL_CARD, null, null, null));
                request.setAttribute("visibleFuncs", func);
                // Excel
                hidd.add( new LocalJavascriptFunctionItem("ReportXls.excelExport", "/images/newdesign/exit_104_darkgray.png", "exportDataGridRows",  RoleAction.EditFuelCards, new AssetSetting[]{ AssetSetting.FUEL_CARD}, "'ChargingTagOverview'"));

                hidd.add( new LocalJavascriptFunctionItem("Register.ChargingProvider", "/images/newdesign/bachelorImages/RfidMassFuncImg.png", "registerRFID",  RoleAction.EditFuelCards, new AssetSetting[]{ AssetSetting.FUEL_CARD}, "'ChargingTagOverview'"));
                request.setAttribute("functionList",hidd);
            }
        } catch (CarAdminException e) {
            getLogger().error("Klarte ikke hente rettigheter",e);
        }
    }

    @NotNull
    private ModernReportFilter getCheckBoxFilter(ReportParametersModern reportParas) {
        ModernReportFilter checkboxfilter;
        List<ModernReportFilter> filters = reportParas.getReportFilters();
        checkboxfilter = new ModernReportFilter();
        checkboxfilter.setHtmlId("columnCheckboxFilter");
        checkboxfilter.setTag(ModernReportFilter.FilterTag.COLUMNCHECKBOX);
        checkboxfilter.setName(m.t("Columns"));
        ArrayList<ColumnCheckbox> colcbs = new ArrayList<>();
        colcbs.add(new ColumnCheckbox("rfidNumberCb", m.t("ChargingTag.ShowRFID"), true, "rfidNumber"));
        colcbs.add(new ColumnCheckbox("descriptionCb", m.t("ChargingTag.showDescription"), true, "description"));
        colcbs.add(new ColumnCheckbox("oilcompanyCb", m.t("ChargingTag.showChargeCompany"), true, "customer.orgname"));
        colcbs.add(new ColumnCheckbox("assewtCb", m.t("ChargingTag.bindAsset"), true, "assets"));
        colcbs.add(new ColumnCheckbox("usersCb", m.t("ChargingTag.bindUser"), false, "users"));
        colcbs.add(new ColumnCheckbox("createdbyCb", m.t("ChargingTag.showCreatedBy"), true, "createdby"));
        colcbs.add(new ColumnCheckbox("createdCb", m.t("ChargingTag.showCreated"), true, "created"));

        checkboxfilter.setColumnCheckBoxes(colcbs);
        filters.add(checkboxfilter);
        return checkboxfilter;
    }

    @Override
    protected ValueProvider getPageTitleProvider() {
        return new TranslatedStringValueProvider(m, "ChargingTag.ChargingTags");
    }

    @Override
    protected ValueProvider getPageHeaderProvider() {
        return new TranslatedStringValueProvider(m, "ChargingTag.ChargingTags");
    }

    @Override
    protected ValueProvider getViewPathProvider() {
        return new StringValueProvider("/view/charging/chargingTagOverview.jsp");
    }
}
