/*
 * Created on Jun 3, 2005
 */
package etc.io.expence;

import etc.caradmin.io.CarAdminException;
import etc.caradmin.io.sql.SQLFactory;
import etc.caradmin.mvc.controller.charging.ChargingProvider;
import etc.caradmin.mvc.controller.charging.ChargingTag;
import etc.caradmin.mvc.controller.charging.ChargingSessionData;


import etc.util.DateTime;
import etc.util.crypto.EtcCrypto;
import etc.util.json.JSONObject;
import org.apache.commons.lang3.StringUtils;
import org.intellij.lang.annotations.Language;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author Rihards D. U.
 */
public class ChargeSQLIO extends ChargeIO {

	public ChargeSQLIO(SQLFactory factory) {
		super(factory);
	}

    /**
     * Hvis ladebrikken har en id, oppdater den, ellers leg den til i databasen.
     *
     * @param ct ladebrikke objektet som blir lagret
     */
    @Override
    public void saveChargingTag(ChargingTag ct) throws CarAdminException {
        // Setter sql queryen til å lagre ladebrikke i databasen
        String sql = "INSERT INTO chargingtag (rfidNumber,description,created,createdby) VALUES(?,?,NOW(),?)";
        // Hvis ladebrikken har en id blir queryen byttet ut med å oppdatere den eksisterende ladebrikken
        if(ct.getId() > 0) {
            sql = "UPDATE chargingtag SET rfidNumber=?,description=? WHERE id = ?";
        }

        int idx = 1;
        // Forsøker å koble til databasen
        try(Connection c = sqlFactory.getConnection() ) {
            try(PreparedStatement p = c.prepareStatement(sql, Statement.RETURN_GENERATED_KEYS)) {
                // Legger til dataen fra ladebrikke objektet for pushing til databasen
                p.setString(idx++,ct.getRfidNumber());
                p.setString(idx++,ct.getDescription());
                if(ct.getId()>0){
                    p.setInt(idx,ct.getId());
                }else{
                    p.setInt(idx,ct.getCreatedBy());
                }
                // Utfører oppdateringen
                p.executeUpdate();
            }
        }
        // Hånterer error ved lagring og sriver ut en feilmelding i konsolen
        catch(SQLException sqle) {
            throw new CarAdminException("Feil ved lagring av ladebrikke: " + sql,sqle);
        }
    }

    @Override
    public ChargingTag getChargingTagById(int id) throws CarAdminException {
	    String sql = "SELECT * FROM chargingtag WHERE id = ?";
        try(Connection c = sqlFactory.getConnection() ) {
            try (PreparedStatement p = c.prepareStatement(sql)) {
                p.setInt(1, id);
                try (ResultSet rs = p.executeQuery()) {
                    if (rs.first()) {
                        return readChargingTag(rs);
                    }
                }
            }
        }catch (SQLException e){
            throw new CarAdminException("Feil i uthenting av ladebrikke",e );
        }
        return null;
    }

    @Override
    public Map<String, String> getChargingTagAndCarIdById(int id) throws CarAdminException {
        String sql = "select rfidNumber, carID from chargingtag inner join chargingtagbinding on chargingtag.id = chargingtagbinding.tagid inner join general on chargingtagbinding.assetno = general.carno WHERE chargingtag.id = ?";
        try(Connection c = sqlFactory.getConnection() ) {
            try (PreparedStatement p = c.prepareStatement(sql)) {
                p.setInt(1, id);
                try (ResultSet rs = p.executeQuery()) {
                    if (rs.first()) {
                        String rfidNumber = rs.getString("rfidNumber");
                        String carID = rs.getString("carID");

                        Map<String, String> rfidCar = new HashMap<>();
                        rfidCar.put("rfidNumber", rfidNumber);
                        rfidCar.put("carID", carID);
                        return rfidCar;
                    }
                }
            }
        }catch (SQLException e){
            throw new CarAdminException("Feil i uthenting av ladebrikke",e );
        }
        return null;
    }

    @Override
    public ChargingTag getChargingTagByNumber(String number) throws CarAdminException {
        String sql = "SELECT * FROM chargingtag WHERE rfidNumber = ?";
        try(Connection c = sqlFactory.getConnection() ) {
            try (PreparedStatement p = c.prepareStatement(sql)) {
                p.setString(1, number);
                try (ResultSet rs = p.executeQuery()) {
                    if (rs.first()) {
                        return readChargingTag(rs);
                    }
                }
            }
        }catch (SQLException e){
            throw new CarAdminException("Feil i uthenting av ladebrikke",e );
        }
        return null;
    }

    public ArrayList<ChargingProvider> getChargingProviders() throws CarAdminException {
        String sql = "SELECT * FROM chargingprovider";
        try(Connection c = sqlFactory.getConnection() ) {
            try (PreparedStatement p = c.prepareStatement(sql)) {
                try (ResultSet rs = p.executeQuery()) {
                    if (rs.first()) {
                        ArrayList<ChargingProvider> result= new ArrayList<>();
                        do{
                            result.add(readChargingProvider(rs));
                        }while (rs.next());
                        return result;
                    }

                }
            }
        }catch (SQLException e){
            throw new CarAdminException("Feil i uthenting av ladeleverand�rer",e );
        }
        return null;
    }



    public List<JSONObject> getChargingProvidersByStatus(int status) throws CarAdminException {
        String sql = "SELECT * FROM chargingprovider WHERE STATUS = ?";
        try(Connection c = sqlFactory.getConnection() ) {
            try (PreparedStatement p = c.prepareStatement(sql)) {
                p.setInt(1, status);
                try (ResultSet rs = p.executeQuery()) {
                    if (rs.first()) {
                        List<JSONObject> result= new ArrayList<>();
                        do{
                            JSONObject j = new JSONObject();
                            j.put("name",rs.getString("name"));
                            j.put("username",rs.getString("username"));
                            j.put("password",rs.getString("password"));
                            j.put("token",rs.getString("token"));
                            result.add(j);

                        }while (rs.next());
                        return result;
                    }

                }
            }
        }catch (SQLException e){
            throw new CarAdminException("Feil i uthenting av ladelevrandører brukeren har lagt til login info til.",e );
        }
        return null;
    }

    /**
     * Henter data om en lading fra databasen basert på id til ladingen.
     * @param id
     * @return
     * @throws CarAdminException
     */
    @Override
    public ChargingSessionData getChargingSessionById(final int id) throws CarAdminException {
        String sql = "SELECT chargerId, date, brand, price, kwh FROM charging WHERE id = ?";
        try(Connection c = sqlFactory.getConnection() ) {
            try (PreparedStatement p = c.prepareStatement(sql)) {
                p.setInt(1, id);
                try (ResultSet rs = p.executeQuery()) {

                    if (rs.first()) {
                        ChargingSessionData chs = new ChargingSessionData();
                        chs.setChargerId(rs.getString("chargerId"));
                        chs.setDate(rs.getString("date"));
                        chs.setBrand(rs.getString("brand"));
                        chs.setPrice(rs.getFloat("price"));
                        chs.setKwh(rs.getFloat("kwh"));
                        return chs;
                    }
                }
            }
        }catch (SQLException e){
            throw new CarAdminException("Feil i uthenting av ladinger",e );
        }

        return null;
    }


       /* try(Connection c = sqlFactory.getConnection() ) {
            try (PreparedStatement p = c.prepareStatement(sql)) {
                p.setInt(1, status);
                try (ResultSet rs = p.executeQuery()) {
                    if (rs.first()) {
                        List<JSONObject> result= new ArrayList<>();
                        do{
                            JSONObject j = new JSONObject();
                            j.put("name",rs.getString("name"));
                            j.put("username",rs.getString("username"));
                            j.put("password",rs.getString("password"));
                            j.put("token",rs.getString("token"));
                            result.add(j);

                        }while (rs.next());
                        return result;
                    }

                }
            }
        }catch (SQLException e){
            throw new CarAdminException("Feil i uthenting av ladelevrandører brukeren har lagt til login info til.",e );
        }
        return null;
    }*/


    public ChargingProvider getChargingProviderByName(String provider) throws CarAdminException {
        String sql = "SELECT * FROM chargingprovider WHERE name = ?";
        try(Connection c = sqlFactory.getConnection() ) {
            try (PreparedStatement p = c.prepareStatement(sql)) {
                p.setString(1, provider);
                try (ResultSet rs = p.executeQuery()) {
                    if (rs.first()) {
                        return readChargingProvider(rs);
                    }

                }
            }
        }catch (SQLException e){
            throw new CarAdminException("Feil i uthenting av ladeleverand�r",e );
        }
        return null;
    }

    public ChargingProvider getChargingProviderById(int provider) throws CarAdminException {
        String sql = "SELECT * FROM chargingprovider WHERE id = ?";
        try(Connection c = sqlFactory.getConnection() ) {
            try (PreparedStatement p = c.prepareStatement(sql)) {
                p.setInt(1, provider);
                try (ResultSet rs = p.executeQuery()) {
                    if (rs.first()) {
                        return readChargingProvider(rs);
                    }

                }
            }
        }catch (SQLException e){
            throw new CarAdminException("Feil i uthenting av ladeleverand�r",e );
        }
        return null;
    }






    private ChargingTag readChargingTag(ResultSet rs) throws SQLException {
        ChargingTag ct = new ChargingTag();
        ct.setId(rs.getInt("id"));
        ct.setCreatedBy(rs.getInt("createdby"));
        String crDate = rs.getString("created");

        if(StringUtils.isNotBlank(crDate)) {
            ct.setCreated(DateTime.fromSql(crDate));
        }

        ct.setRfidNumber(rs.getString("rfidNumber"));
        ct.setDescription(rs.getString("description"));

        return ct;
    }

    private ChargingProvider readChargingProvider(ResultSet rs) throws SQLException {
        ChargingProvider cp = new ChargingProvider();
        EtcCrypto crypto = new EtcCrypto();

        String decryptedPass = crypto.decrypt(rs.getString("password"));
        cp.setPassword(decryptedPass);
        cp.setId(rs.getInt("id"));
        cp.setName(rs.getString("name"));
        cp.setUserName(rs.getString("userName"));
        if(StringUtils.isNotBlank(cp.getUserName()) && StringUtils.isNotBlank(cp.getPassword())){
            cp.setRegistered(true);
        }else cp.setRegistered(false);
        cp.setToken(rs.getString("token"));

        return cp;
    }

    @Override
    public void addChargingTagBinding(int tagId, int assetNo, int userId) throws CarAdminException {
        String check = "SELECT * FROM chargingtagbinding WHERE tagId = ? AND assetNo = ? AND userId = ?";

        @Language("SQL")
        String sql = "INSERT INTO chargingtagbinding (tagid, assetno, userid) VALUES(?,?,?)";
        try(Connection c = sqlFactory.getConnection()) {
            try(PreparedStatement che = c.prepareStatement(check)) {
                che.setInt(1, tagId);
                che.setInt(2, assetNo);
                che.setInt(3, userId);
                try (ResultSet rs = che.executeQuery()) {
                    if (!rs.first()) {
                        sqlFactory.executeUpdate(c,sql,tagId,assetNo,userId);
                    }
                }
            }
        }
        catch(SQLException sqle) {
            throw new CarAdminException("Feil ved lagring av brikke binding: " + sql,sqle);
        }
    }

    @Override
    public void addChargingProviderBinding(int tagId, int providerid, int status) throws CarAdminException {
        String check = "SELECT * FROM chargingproviderbinding WHERE tagId = ? AND providerid = ?";

        @Language("SQL")
        String sqlIns = "INSERT INTO chargingproviderbinding (tagid, providerid, status) VALUES(?,?,?)";
        @Language("SQL")
        String sqlUp  = "UPDATE chargingproviderbinding SET status = ? WHERE tagid = ? AND providerid = ?";
        try(Connection c = sqlFactory.getConnection()) {
            try(PreparedStatement che = c.prepareStatement(check)) {
                che.setInt(1, tagId);
                che.setInt(2, providerid);
                try (ResultSet rs = che.executeQuery()) {
                    if (!rs.first()) {
                        sqlFactory.executeUpdate(c,sqlIns,tagId,providerid, status);
                    }else{
                        sqlFactory.executeUpdate(c, sqlUp, status, tagId, providerid);
                    }
                }
            }
        }
        catch(SQLException sqle) {
            throw new CarAdminException("Feil ved lagring av ladeleverand�r binding: " + sqlIns,sqle);
        }
    }

    @Override
    public void removeChargingTagBinding(int tagId, int assetNo, int userId) throws CarAdminException {
        String sql = "DELETE from chargingtagbinding WHERE tagid = ? AND assetNo = ? AND userId = ?";
        try(Connection c = sqlFactory.getConnection() ) {
            try (PreparedStatement p = c.prepareStatement(sql)) {
                p.setInt(1, tagId);
                p.setInt(2, assetNo);
                p.setInt(3, userId);
                p.executeUpdate();
            }
        }
        catch(SQLException sqle) {
            throw new CarAdminException("Feil ved lagring av brikke binding: " + sql,sqle);
        }
    }

    @Override
    public void removeChargingProviderBinding(int tagId, int providerid) throws CarAdminException {
        String sql = "DELETE from chargingproviderbinding WHERE tagid = ? AND providerid = ?";
        try(Connection c = sqlFactory.getConnection() ) {
            try (PreparedStatement p = c.prepareStatement(sql)) {
                p.setInt(1, tagId);
                p.setInt(2, providerid);
                p.executeUpdate();
            }
        }
        catch(SQLException sqle) {
            throw new CarAdminException("Feil ved fjerning av ladeleverand�r binding: " + sql,sqle);
        }
    }

    @Override
    public List<JSONObject> getBindingsForTag(int tagId) throws CarAdminException {
        String sql = "SELECT * FROM chargingtagbinding " +
                "LEFT JOIN general on general.carno = chargingtagbinding.assetno " +
                "LEFT JOIN user on user.userid = chargingtagbinding.userid " +
                "WHERE chargingtagbinding.tagid = ?";

        List<JSONObject> bindings = new ArrayList<>();
        try(Connection c = sqlFactory.getConnection() ) {
            try (PreparedStatement p = c.prepareStatement(sql)) {
                p.setInt(1, tagId);
                try (ResultSet rs = p.executeQuery()) {
                    if (rs.first()) {
                        do{
                            JSONObject j = new JSONObject();
                            j.put("tagid",rs.getInt("tagid"));
                            int assetNo = rs.getInt("assetno");
                            int userId = rs.getInt("userid");
                            if(assetNo > 0){
                                String carId = rs.getString("carid");
                                if(StringUtils.isNotBlank("carid")){
                                    j.put("assetno",assetNo);
                                    j.put("carid", carId);
                                }
                            }
                            if(userId > 0){
                                String first = rs.getString("firstname");
                                String surname = rs.getString("surname");
                                if(StringUtils.isNotBlank(first) && StringUtils.isNotBlank(surname)){
                                    j.put("userid",userId);
                                    j.put("username",first+" "+surname);
                                }
                            }
                            bindings.add(j);
                        }while (rs.next());
                    }
                }
            }
        }catch (SQLException e){
            throw new CarAdminException("Feil i uthenting av ladebrikke",e );
        }
        return bindings;
    }

    public List<JSONObject> getBindingsForProvider(int tagId) throws CarAdminException {
        String sql = "SELECT * FROM chargingproviderbinding " +
                "LEFT JOIN chargingprovider on chargingprovider.id = chargingproviderbinding.providerid " +
                "WHERE chargingproviderbinding.tagid = ?";

        List<JSONObject> bindings = new ArrayList<>();
        try(Connection c = sqlFactory.getConnection() ) {
            try (PreparedStatement p = c.prepareStatement(sql)) {
                p.setInt(1, tagId);
                try (ResultSet rs = p.executeQuery()) {
                    if (rs.first()) {
                        do{
                            JSONObject j = new JSONObject();
                            j.put("tagid",rs.getInt("tagid"));
                            int providerId = rs.getInt("providerid");
                            int status = rs.getInt("status");
                            if(providerId > 0){
                                String name = rs.getString("name");
                                if(StringUtils.isNotBlank("name")){
                                    j.put("providerid", providerId);
                                    j.put("status", status);
                                    j.put("name", name);
                                }
                            }
                            bindings.add(j);
                        }while (rs.next());
                    }
                }
            }
        }catch (SQLException e){
            throw new CarAdminException("Feil i uthenting av ladebrikkeleverand�r",e );
        }
        return bindings;
    }

    @Override
    public void updateChargingProviderToken(final String provider, final String token) throws CarAdminException {
        @Language("SQL")
        String sql = "UPDATE chargingprovider SET token = ? WHERE NAME = ?";
        try(Connection c = sqlFactory.getConnection()) {
            try(PreparedStatement ps = c.prepareStatement(sql)) {
                ps.setString(1, token);
                ps.setString(2, provider);
                ps.executeUpdate();

            }
        }
        catch(SQLException sqle) {
            throw new CarAdminException("Feil ved oppdatering av chargingprovider token: " + sql,sqle);
        }
    }

    @Override
    public void updateChargingProviderUserInfo(final String username, final String password, final String provider, int status) throws CarAdminException {
        @Language("SQL")
        String sql = "UPDATE chargingprovider SET username = ?, password = ?, status = ? WHERE NAME = ?";

        try(Connection c = sqlFactory.getConnection()) {
            try(PreparedStatement ps = c.prepareStatement(sql)) {
                ps.setString(1, username);
                ps.setString(2, password);
                ps.setInt(3, status);
                ps.setString(4, provider);
                ps.executeUpdate();
            }
        }
        catch(SQLException sqle) {
            throw new CarAdminException("Feil ved oppdatering av chargingprovider userName og Password: " + sql,sqle);
        }
    }

}
