package etc.io.logging.expence;

import etc.beans.gdpr.GdprPrivacyContext;
import etc.beans.user.SolutionSessionInfo;
import etc.caradmin.io.CarAdminException;
import etc.caradmin.io.sql.SQLFactory;
import etc.io.expence.ChargeIO;
import etc.io.gdpr.LoggingIO;
import etc.io.ClassFactory;
import etc.io.expence.FuelIO;

/**
 * Wrapper-klasse for {@link FuelIO} som utfrer logging tilknyttet GDPR.
 *
 * @author Anders Engen Olsen
 */
public abstract class LoggingChargingIO extends LoggingIO {

    /**
     * Databaseaksess
     */
    ChargeIO chargeIO;

    LoggingChargingIO(SQLFactory sqlFactory) {
        super(sqlFactory);
        this.chargeIO = ClassFactory.getChargeIO(sqlFactory);
    }

    /**
     * Legger til en knytting for ladebrikke, og logger tilknytningen dersom en bruker blir knyttet.
     *
     * @param tagId  id p kort
     * @param assetNo id p kjrety
     * @param userId  id p bruker
     * @param si      sesjonsobjekt
     * @param context Kontekst for endring
     * @throws CarAdminException hvis io feiler
     */
    public abstract void addChargingTagBinding(int tagId, int assetNo, int userId, SolutionSessionInfo si, GdprPrivacyContext context) throws CarAdminException;

    /**
     * Fjerner en knytning for ladebrikke, og logger dersom tilknytning som fjernes er en bruker.
     * @param tagId id p ladebrikke
     * @param assetNo id p kjrety
     * @param userId id p bruker
     * @param si sesjonsobjekt
     * @param context Kontekst for endring
     * @throws CarAdminException hvis io feiler
     */
    public abstract void removeChargingTagBinding(int tagId, int assetNo, int userId, SolutionSessionInfo si, GdprPrivacyContext context) throws CarAdminException;

    /**
     * Fjerner en knytning for ladeselskap, og logger dersom tilknytning som fjernes er en bruker.
     * @param tagId id p ladebrikke
     * @param providerid id p ladeleverandr
     * @param si sesjonsobjekt
     * @param context Kontekst for endring
     * @throws CarAdminException hvis io feiler
     */
    public abstract void removeChargingProviderBinding(int tagId, int providerid, SolutionSessionInfo si, GdprPrivacyContext context) throws CarAdminException;

}
