package Functional.Charging;

import etc.ajax.chargingTagProviderParsing.ChargingTagProviderParser;
import org.junit.jupiter.api.Test;

import java.io.IOException;

import static org.junit.jupiter.api.Assertions.assertNotEquals;
import static org.junit.jupiter.api.Assertions.*;

public class ChargingProviderTest {

    ChargingTagProviderParser parser = new ChargingTagProviderParser();

    @Test
    void testEonLogin() {
        try {
            String token = parser.eonLogin("aaalsaye@stud.ntnu.no", "Bachelor12345!");
            assertNotEquals("", token);
        } catch(IOException e) {
            e.printStackTrace();
        }
    }

    @Test
    void testBilkraftLogin() {
        try {
            String token = parser.bilkraftLogin("469192991", "bachelor12345!");
            assertNotEquals("", token);
            assertNotEquals(null, token);
        } catch(IOException e) {
            e.printStackTrace();
        }
    }

    @Test
    void testBilkraftGetLadebrikker() {
        try {
            String token = parser.bilkraftLogin("46919299","bachelor12345!");
            assertNotEquals(null, parser.bilkraftGetLadebrikker(token));
        } catch(IOException e) {
            e.printStackTrace();
        }
    }

    @Test
    void testEonGetLadebrikker() {
        try {
            // Henter token, som blir brukt til å hente ut alle ladebrikker
            String token = parser.eonLogin("aaalsaye@stud.ntnu.no","Bachelor12345!");
            // Sjekker at vi fikk ut noen ladebrikker
            assertNotEquals(null, parser.eonGetLadebrikker(token));
        } catch(IOException e) {
            e.printStackTrace();
        }
    }

    @Test
    void testBilkraftPostAndDeleteLadebrikker() {
        try {
            // Henter token, som blir brukt til å legge til, og slette ladebrikke.
            String token = parser.bilkraftLogin("46919299", "bachelor12345!");
            // Kaller på funksjonen som legger til ladebrikke.
            testBilkraftPostLadebrikke(token, "123444441");
            // Kaller på funksjonen som sletter ladebrikke. Sletter samme ladebrikke som nettop ble lagt inn.
            testBilkraftDeleteLadebrikker(token, "123444441");
        } catch(IOException e) {
            e.printStackTrace();
        }

    }

    void testBilkraftPostLadebrikke(String token, String rfidnummer) {
        try {
            // Sender inn rfidnummer og bilnummer til Bilkraft, for å registrere en ny ladebrikke knyttet til bilen.
            String testRes = parser.bilkraftPost(rfidnummer, "FT 16778", token);
            // Sjekker at registrering av ladebrikke ikke feilet.
            assertNotEquals("FAIL", testRes);
            if(!testRes.equals("FAIL")) {
                System.out.println(rfidnummer + " : POST suksess - bilkraft");
            }
        } catch(IOException e) {
            e.printStackTrace();
        }
    }

    void testBilkraftDeleteLadebrikker(String token, String rfidnummer) {
        try {
            // Sender inn rfidnummer til brikken vi ønsker  slette.
            String testRes = parser.bilkraftDelete(rfidnummer, token);
            // Sjekker om sletting var suksessfull.
            assertEquals("success", testRes);
            if(testRes.equals("success")) {
                System.out.println(rfidnummer + " : DELETE suksess - bilkraft");
            }
        } catch(IOException e) {
            e.printStackTrace();
        }
    }

    @Test
    void testEonPostAndDeleteLadebrikker() {
        try {
            // Henter token, som vi bruker til å legge til ny ladebrikke.
            String token = parser.eonLogin("aaalsaye@stud.ntnu.no", "Bachelor12345!");
            // Kaller på funksjonen som legger til ladebrikke.
            testEonPostLadebrikke(token, "123444441");
            // Kaller på funksjonen som sletter ladebrikke. Sletter samme ladebrikke som nettop ble lagt inn.
            testEonDeleteLadebrikker(token, "123444441");
        } catch(IOException e) {
            e.printStackTrace();
        }
    }

    void testEonPostLadebrikke(String token, String rfidnummer) {
        try {
            // Sender inn rfidnummer til EON, for å registrere en ny ladebrikke.
            String testRes = parser.eonPost(rfidnummer, token);
            // Sjekker at registrering av ladebrikke ikke feilet.
            assertNotEquals("FAIL", testRes);
            if(!testRes.equals("FAIL")) {
                System.out.println(rfidnummer + " : POST suksess - EON");
            }
        } catch(IOException e) {
            e.printStackTrace();
        }

    }

    void testEonDeleteLadebrikker(String token, String rfidnummer) {
        try {
            // Sender inn rfidnummer til brikken vi ønsker å slette.
            String testRes = parser.eonDelete(rfidnummer, token);
            // Sjekker om sletting var suksessfull.
            assertEquals("success", testRes);
            if(testRes.equals("success")) {
                System.out.println(rfidnummer + " : DELETE suksess - EON");
            }
        } catch(IOException e) {
            e.printStackTrace();
        }
    }

    @Test
    void testEonPostAndDelete100Ladebrikker() {
        try {
            // Henter token, som vi bruker til å legge til ny ladebrikke.
            String token = parser.eonLogin("aaalsaye@stud.ntnu.no", "Bachelor12345!");
            String rfidnummer = "";

            for(int i = 0; i < 100; i++) {
                rfidnummer = "123444441";
                rfidnummer += i;
                // Kaller på funksjonen som legger til ladebrikke.
                testEonPostLadebrikke(token, rfidnummer);
            }

            for(int i = 0; i < 100; i++) {
                rfidnummer = "123444441";
                rfidnummer += i;
                // Kaller på funksjonen som sletter ladebrikke. Sletter samme ladebrikker som nettop ble lagt inn.
                testEonDeleteLadebrikker(token, rfidnummer);
            }

        } catch(IOException e) {
            e.printStackTrace();
        }
    }

    @Test
    void testBilkraftPostAndDelete100Ladebrikker() {
        try {
            // Henter token, som vi bruker til å legge til ny ladebrikke.
            String token = parser.bilkraftLogin("46919299","bachelor12345!");
            String rfidnummer = "";

            for(int i = 0; i < 100; i++) {
                rfidnummer = "123444441";
                rfidnummer += i;
                // Kaller på funksjonen som legger til ladebrikke.
                testBilkraftPostLadebrikke(token, rfidnummer);
            }

            for(int i = 0; i < 100; i++) {
                rfidnummer = "123444441";
                rfidnummer += i;
                // Kaller på funksjonen som sletter ladebrikke. Sletter samme ladebrikker som nettop ble lagt inn.
                testBilkraftDeleteLadebrikker(token, rfidnummer);
            }

        } catch(IOException e) {
            e.printStackTrace();
        }
    }

}

